;  Test program for TORANDO-31 processor.

;****************** Interrupt & init vectors zone **********************

              .sect "vectors"
init_vector .word start_prog   ; program start address

;***********************************************************************


;************************* Main Program zone ***************************



;==================== vars & constants definition ======================
        .data
MEM_VARS:

start_mem_ptr       .word  100h       ; memory pointer for WRITE-TEST
start_data_mem      .word  97513fdbh    ; start data to be stored in memory
last_mem_ptr        .word  8000h    ; last memory pointer
data_inc		.word 13579bdfh;

stack_zone          .word  87fe00h    ; start of stack zone (to be incremented)

iof_mode            .set   6          ; IO_Flags mode: IOF0 - out, IOF1 - in

name                .string "my_program"



;-------------------- Cache control ------------------------------------
__cache_clear       .set   1000h   ; cache clear bit
__cache_en          .set   800h    ; cache enable bit
__cache_freeze      .set   400h    ; cache freeze bit

CACHE_EN          $MACRO           ; macro for CACHE CLEAR/UNFREEZE/ENABLE
             and ~(__cache_en | __cache_freeze),st
             or  __cache_clear,st
             or  __cache_en,st
                  $ENDM

CACHE_DISABLE     $MACRO           ; macro for CACHE DISABLE
             and ~__cache_en,st
                  $ENDM

CACHE_FREEZE      $MACRO           ; macro for CACHE FREEZE
             or __cache_freeze,st
                  $ENDM

CACHE_UNFREEZE    $MACRO           ; macro for CACHE UNFREEZING
             and ~__cache_freeze,st
                  $ENDM
;--------------------------------------------------------------------------

;----------------- Interrupts control -------------------------------------
__gint_en        .set   2000h      ; global INTERRUPT ENABLE bit

INT_ENABLE       $MACRO           ; macro for INTERRUPTs ENABLE
             or  __gint_en,st
                 $ENDM

INT_DISABLE      $MACRO           ; macro for INTERRUPTs DISABLE
             and ~__gint_en,st
                 $ENDM
;--------------------------------------------------------------------------



;---------------------- I/O configuration & vars --------------------------

PERIPH_ADDR         .set   800000h ; C30 memory mapped peripherals base
                                   ; address (for DP register)

pbus_cntr_rg        .set   808064h ; C30 Primary Bus Control reg addr
pbus1_cntr_rg       .set   808068h ; C30 Primary Bus Control reg addr
pbus_cntr           .word  0f0600h    ; RDY= ext_RDY, BNKCMP=6 (256K)

xbus_cntr_rg        .set   808060h ; C30 Expansion Control reg addr
xbus_cntr           .set   018h    ; SWW=11 (PXRDY only),

; C30 Timer#0 is used to generate time control clocks for the time controller
; of the BiPS/SBUS accesses. The clock must be 500..600ns to provide maximum
; SBUS access time at 8us rate.

tm0_cntr_rg         .set   808020h ; Timer #0 global control address
tm0_period_rg       .set   808028h ; Timer #0 period address
tm0_cntr            .set   601h    ; contol: PULSE/INVERT, CLKSRC=internal
tm_GO               .set   0c0h    ; OR this to CONTROL for timer START
tm0_period          .set   5
;-------------------------------------------------------------------------




; ------- external interrupts_requests & strobes generator ---------------
; (connected to C30 expansion BUS)

__irq_to_pc_addr    .word  804000h ; external SYNCH #0 to form IBM PC/AT IRQ
__irq_to_dsp_addr   .word  804001h ; external SYNCH #1 to form other DSP IRQ
__aux_strobe_1_addr .word  804002h ; aux SYNCH #1
__aux_strobe_2_addr .word  804003h ; aux SYNCH #2


IRQ_PC            $MACRO           ; macro for the IRQ_TO_PC generator
             sti r0,@__irq_to_pc_addr
                  $ENDM

IRQ_DSP           $MACRO           ; macro for the IRQ_TO_DSP generator
             sti r0,@__irq_to_dsp_addr
                  $ENDM

AUX_STB1          $MACRO           ; macro for the AUX STROBE #1 generation
             sti r0,@__aux_strobe_1_addr
                  $ENDM

AUX_STB2          $MACRO           ; macro for the AUX STROBE #2 generation
             sti r0,@__aux_strobe_2_addr
                  $ENDM

;---------------------------------------------------------------------------
;===========================================================================


;============================= program ===============================

        .text

start_prog:
        ldp MEM_VARS       ; set memory page
        ldi @stack_zone,sp

        CACHE_DISABLE
;        INT_DISABLE
        CACHE_EN

        call io_set               ; configure I/O

;------------------------ memory testing --------------------------
        ldi 0,r7                  ; r7 - counter of successful passes
        ldi @start_data_mem,r2
	ldi @data_inc,r6

mem_test_start:
        ldi @start_mem_ptr,ar0    ; set pointer/data for start memory address
        ldi r2,r1                 ; set start memory pattern

                                  ; memory write loop
wr_loop:
           sti r1,*ar0++          ; simple memory writing
	   addi 1,r1
	   cmpi @last_mem_ptr,ar0
	   bc wr_loop

        ldi @start_mem_ptr,ar0    ; set pointer/data for start memory address
        ldi r2,r1
                                  ; memory verify loop
ver_loop:
           ldi *ar0++,r3          ; read memory
           cmpi r1,r3             ; verify memory
	   bnz  $_err_detected
	   addi 1,r1
	   cmpi @last_mem_ptr,ar0
	   bc ver_loop

        addi 1,r7                 ; increment counter of successful passes
        addi r6,r2
        b mem_test_start          ; repeat test again


$_err_detected:
           b  $_err_detected      ; iddle here on error

;-----------------------------------------------------------------------

;-------------------- C30 I/O configuring procedure ---------------------

io_set:
        push dp

        ldp MEM_VARS       ; set memory page
        ldi @pbus_cntr,ar7        ; set C30 Primary BUS parameters

        ldp PERIPH_ADDR          ; switch to memory mapped periph.page

        sti ar7,@pbus_cntr_rg
        sti ar7,@pbus1_cntr_rg

;        ldi xbus_cntr,ar7        ; set C30 Expansion BUS parameters
;        sti ar7,@xbus_cntr_rg

               ;----------- programming the TM#0 for S-BUS time control
        ldi 0,ar7                ; set TM#0 mode
        sti ar7,@tm0_cntr_rg     ; clear timer mode
        ldi tm0_period,ar7       ; set TM#0 period
        sti ar7,@tm0_period_rg
        ldi tm0_cntr,ar7         ; set TM#0 mode
        sti ar7,@tm0_cntr_rg
        or  tm_GO,ar7
        sti ar7,@tm0_cntr_rg

        ldi iof_mode,iof         ; configure I/OF-0 as an output, IOF-1 as input

        pop dp
        rets
;------------------------------------------------------------------------



;************************************************************************



;******************** overlaid data structures **************************

      .sect "mcd_par"

mcd_par         .struct
par1_par2   .word
par3        .word
par4        .float
               .endstruct


      .sect "mcd_mlcp"
mcd_mlcp        .struct
p_sptr           .word
p_len            .word
r_sptr           .word
r_len_max        .word
r_len            .word
tea_tcs_rea_rcs  .word
                .endstruct


;************************************************************************



	.end
