/************************************************************************/
/*                                                                      */
/*              Copyright 1990 by Accelerated Technology                */
/*                                                                      */
/*  PROPRIETARY RIGHTS of Accelerated Technology are involved in the    */
/*  subject matter of this material.  All manufacturing, reproduction,  */
/*  use, and sales rights pertaining to this subject matter are         */
/*  governed by the license agreement.  The buyer or recipient of this  */
/*  package, implicitly accepts the terms of the license.               */
/*                                                                      */
/************************************************************************/

/************************************************************************/
/*                                                                      */
/*  FILE DESCRIPTION                                                    */
/*                                                                      */
/*  This file contains information necessary for the user to configure  */
/*  all of the nucleus resources.                                       */
/*                                                                      */
/*  ROUTINES                                                            */
/*                                                                      */
/*      None                                                            */
/*                                                                      */
/*  NOTES                                                               */
/*                                                                      */
/*      At link time this file should be loaded last.  In this way,     */
/*      a determination can be made as to the last address used.        */
/*                                                                      */
/************************************************************************/

#include              "nu_defs.h"           /* General constants        */
#include              "in_defs.h"           /* Include definitions      */

/* Define the last available memory address in the system.  The variable
   memory area uses the memory between the Last_Memory_Address and the
   last used address.  The last used address includes the allocation
   for all of the instruction areas, data areas, stacks, and the fixed
   partitions.  Note that this address should be valid for an unsigned int
   type C access.  */

unsigned int  IN_Last_Memory_Address = 0x7fff;


/* The following array of structures "IN_Fixed_Partitions[]" defines the
   fixed memory partitions of the run-time system.  Note that partition IDs
   (used in subsequent service calls) correspond to the indices of this
   array.  It is often useful to define constants for the IDs in an
   application include file.  Also note that the size is in terms of
   "unsigned int" words.  */

struct IN_FIXED_PARTITION_STRUCT  IN_Fixed_Partitions[] =
{
    {20, 6},                                /* Example System, 5 partitions
                                               of 20 unsigned ints each  */
                                            /* Add partition definitions:
                                              {partition_size, count}, */
    {END_OF_LIST}                           /* Signifies last item in
                                               the list.               */
};


/* The following array of IN_QUEUE_DEFINITION_STRUCTs "IN_System_Queues[]"
   defines the number of items in each system queue.  An item is defined
   as a multiple of the unsigned int type.  Note that queue IDs (used in
   subsequent service calls) correspond to the indices of this array.
   It is often useful to define constants for the queue IDs.  */

struct IN_QUEUE_DEFINITION_STRUCT  IN_System_Queues[] =
{
    {100, 2},                               /* Example queue of 100 items
                                               that each have 2 words.  */
                                            /* Add queue definitions:
                                               {  number_of_items,
                                                  size_of_item  },      */
    {END_OF_LIST}                           /* Signifies last item in
                                               the list.                */
};


/* The following unsigned int "IN_System_Stack_Size" variable defines the
   system stack size in terms of the number of unsigned ints necessary.  */

unsigned int  IN_System_Stack_Size = 400;


/* The following unsigned int "IN_System_Event_Groups" variable defines the
   total number of system event groups.  System event group IDs range from 0
   through IN_System_Event_Groups - 1.  Again, it is often useful to define
   constants for the Event Group IDs.  */

unsigned int  IN_System_Event_Groups =  1;


/* The following unsigned int "IN_System_Resources" variable defines the
   total number of system resources.  System resource IDs range from 0
   through IN_System_Resources - 1.  Again, it is often useful to define
   constants for the resource IDs.  */

unsigned int  IN_System_Resources =  1;


/* The following array of structures "IN_System_Tasks[]" defines the
   tasks in the system.  Note that task IDs (used in subsequent service calls)
   correspond to the indices of this array.  It is often useful to define
   constants for the task IDs.  Additionally note that a task cannot have
   a starting address of 0.  Yet another comment, the stack size is in terms
   of the number of "unsigned int" types required.  */

extern void   task_0();
extern void   task_1();
extern void   task_2();
extern void   task_3_and_4();
extern void   task_5();

extern void   DBP_Debug();


struct IN_TASK_DEFINITION_STRUCT  IN_System_Tasks[] =
{
      {task_0,       200,  2, NU_TRUE, NU_FALSE, -1},  /* Timer Task      */
      {task_1,       200, 10, NU_TRUE, NU_FALSE,  5},  /* Queue Sender    */
      {task_2,       200, 10, NU_TRUE, NU_FALSE,  5},  /* Queue Receiver  */
      {task_3_and_4, 200,  5, NU_TRUE, NU_FALSE, -1},  /* Resource Alloc  */
      {task_3_and_4, 200,  5, NU_TRUE, NU_FALSE, -1},  /* Resource Alloc  */
      {task_5,       200,  2, NU_TRUE, NU_FALSE, -1},  /* Event flag user */
      {DBP_Debug,    200,  1, NU_TRUE, NU_FALSE,  2},  /* NUK DEBUGger    */

                                            /* Add partition definitions:
                                              {task_address, stack_size,
                                               task_priority (0-255),
                                               automatic_start (TRUE/FALSE),
                                               non-preemptable (TRUE/FALSE),
                                               time_slice (-1 to disable)}, */
    {END_OF_LIST}                           /* Signifies last item in
                                               the list.                */
};


/* The following variable is used simply as an indication of the last
   address used by the linker for instructions, initialized data, and
   un-initialized data.  Nucleus data structures, task stacks, partition
   memory, and variable memory are allocated after the address of this
   variable.  */

unsigned int  IN_Last_Address_Used;
