#include <math.h>
#include "iface.h"
#include "grint.h"
#include "node1.h"

#define PI 3.141592

 typedef struct
   {
   double  x;
   double  y;
   } XYPOS;


 typedef struct
   {
   XYPOS  allpos [8];
   double  f1;
   double  f2;
   } PARMS;


 void ball (int id)
   {
   int        i, j, color;
   FILLSTYLE  F;
   K_MSG      M;
   PARMS      parms;
   XYPOS      mypos;
   double     x, y, vx, vy, fx, fy, m;
   int        R = 6;

   int colors [9] =
     {
     BLACK,
     LIGHTGREEN,
     LIGHTRED,
     LIGHTMAGENTA,
     YELLOW,
     LIGHTBLUE,
     GREEN,
     LIGHTCYAN,
     MAGENTA
     };

   color = colors [id + 1];
   x = 160.0 * sin (id * PI / 4); 
   y = 160.0 * cos (id * PI / 4);
   vx = vy = 0.0;
 
   KS_LockW (GRAPHRES);
   while (1)
     {
     /* Plot ball at current position */
     F.type  = SOLID_FILL;
     F.color = color;
     setfillstyle (&F);
     setcolor (WHITE);
     fillellipse ((int) x + 320, (int) y + 240, R, R);
     KS_Unlock (GRAPHRES);

     for (j = 0; j < 1; j++)
       {
       /* Report current position to master task */

       mypos.x = x;
       mypos.y = y;
       M.size = sizeof (XYPOS);
       M.info = id;
       M.tx_data = &mypos;
       M.rx_task = MASTER;
       KS_SendW (REPORT, 1, &M);

       /* Wait for message from master, giving current
          positions for all balls and parameters */

       M.size = sizeof (PARMS);
       M.rx_data = &parms;
       M.tx_task = MASTER;
       KS_ReceiveW (GETPOS, &M);

       /* Compute mutual forces */

       fx = fy = 0.0;
       for (i = 0; i < 8; i++)
         {
         double dx, dy;
         if (i == id) continue;
         dx = parms.allpos [i].x - x;
         dy = parms.allpos [i].y - y;
         m = dx * dx + dy * dy + 1;
         fx += dx / m;
         fy += dy / m;
         }
       fx *= parms.f1;
       fy *= parms.f1;

       /* Add 'magnetic field' force */

       fx += parms.f2 * vy;
       fy -= parms.f2 * vx;

       /* Calculate new speed vector */

       vx += fx;
       vy += fy;

       /* Limit speed to 30.0 */

       m = sqrt (1 + parms.f2 * parms.f2);
       vx /= m;
       vy /= m;

       /* Calculate new position */

       x += vx;
       y += vy;

       /* Bounce ball on screen limits */

       do
         {
         i = 0;
         if (x < -310.0) { x = -620.0 - x; vx = -vx; i++; }
         if (x  > 310.0) { x =  620.0 - x; vx = -vx; i++; }
         if (y < -230.0) { y = -460.0 - y; vy = -vy; i++; }
         if (y >  230.0) { y =  460.0 - y; vy = -vy; i++; }
         }
       while (i);
       }

     /* Unplot ball at previous position */

/*     F.color = BLACK;*/
     F.type  = SOLID_FILL;
     KS_LockW (GRAPHRES);
     setfillstyle (&F);
     setcolor (DARKGRAY);

     fillellipse ((int) mypos.x + 320, (int) mypos.y + 240, R, R);
     }
   }



