#include <stdlib.h>
#include <math.h>
#include "iface.h"
#include "node1.h"
#include "_stdio.h"
#include "grint.h"


 typedef struct
   {
   double  x;
   double  y;
   } XYPOS;


 typedef struct
   {
   XYPOS   allpos [8];
   double  f1;
   double  f2;
   } PARMS;



#define PRINT(s) puts (s)


 static void printhelp (void)
   {
   PRINT ("\nThis program simulates the action of 8 'charged particles'.");
   PRINT ("The colour of a particle indicates on which node its task executes.");
   PRINT ("\nYou should type two numerical parameters on the command line,");
   PRINT ("e.g. 'run test -30 40'.");
   PRINT ("\nThe first parameter sets the strength of the mutual forces between");
   PRINT ("any two particles, which is also inversely proportional to their");
   PRINT ("distance.");
   PRINT ("The second parameter sets the intensity of a 'magnetic field', that");
   PRINT ("creates a force proportional and perpendicular to the velocity");
   PRINT ("of a particle.");
   PRINT ("\nA VGA screen is required for this demo.");
   KS_Sleep (500);
   server_terminate ();
   }


 void master (void)
   {
   int    i;
   PARMS  parms;
   XYPOS  report;
   K_MSG  M;
   char   arg [20];

   /* get command line arguments, if any */

   if (server_getarg (1, arg, 20)) parms.f1 = atof (arg) / 1.0;
   else
     {
     printhelp ();
     return;
     }

   if (server_getarg (2, arg, 20)) parms.f2 = atof (arg) / 100.0;
   else parms.f2 = 0.0;
   /* Initialise screen */

   KS_LockW (GRAPHRES);
   initgraph (VGA, VGAHI);
   KS_Unlock (GRAPHRES);

   /* Start all balls */
   KS_StartG (BALLS);

   while (1)
     {
     /* Get position report for each ball */

     for (i = 0; i < 8; i++)
       {
       M.size    = sizeof (XYPOS);
       M.rx_data = &report;
       M.tx_task = ANYTASK;
       KS_ReceiveW (REPORT, &M);
       parms.allpos [M.info] = report;
       }

     /* Send list of all positions to all balls */

     for (i = 0; i < 8; i++)
       {
       M.size    = sizeof (PARMS);
       M.tx_data = &parms;
       M.rx_task = ANYTASK;
       KS_SendW (GETPOS, 1, &M);
       }
     }
   }

