#include <string.h>
#include <_stdio.h>
#include "iface.h"
#include "node1.h"


#define FORMAT "%-60s   %5d\n"

 static char  text [4096];
 static K_MSG Message;



 static void message_test (int size)
   {
   int i, t;

   Message.rx_task = ANYTASK;
   Message.tx_data = text;
   Message.size    = size;

   KS_Elapse (&t);
   for (i = 0; i < 1000; i++) KS_SendW (MAILB1, 1, &Message);
   t = KS_Elapse (&t);
   t *= ticktime;
   t /= 1000;

   if (size) printf ("%20d words: %40d\n", size, t);
   else      printf ("%26s :%40d\n", "header only", t);
   KS_Sleep (250);
   }


 static void benchmark (void)
   {
   K_TICKS et;
   int     i;
   void   *p;

   puts ("");
   KS_Sleep (250);

   KS_Elapse (&et);
   for (i = 0; i < 10000; i++) KS_Nop ();
   et = KS_Elapse (&et);
   et *= ticktime;
   et /= 10000;
   printf (FORMAT, "minimum VIRTUOSO call time", et);
   KS_Sleep (250);

   puts ("send message to waiting high priority task and wait");
   KS_Start (RECVTASK);
   KS_Sleep (250);

   message_test (0);
   message_test (8);
   message_test (16);
   message_test (32);
   message_test (64);
   message_test (128);
   message_test (256);
   message_test (512);
   message_test (1024);
   message_test (2048);
   message_test (4096);


   KS_Elapse (&et);
   for (i = 0; i < 500; i++) KS_EnqueueW (DEMOQX1, text, 1);
   et = KS_Elapse (&et);
   et *= ticktime;
   et /= 500;
   printf (FORMAT, "enqueue 1 word", et);
   KS_Sleep (250);

   KS_Elapse (&et);
   for (i = 0; i < 500; i++) KS_DequeueW (DEMOQX1, text ,1);
   et = KS_Elapse (&et);
   et *= ticktime;
   et /= 500;
   printf (FORMAT, "dequeue 1 word", et);
   KS_Sleep (250);

   KS_Elapse (&et);
   for (i = 0; i < 500; i++) KS_EnqueueW (DEMOQX4, text ,4);
   et = KS_Elapse (&et);
   et *= ticktime;
   et /= 500; /* convert to us */
   printf (FORMAT, "enqueue 4 words", et);
   KS_Sleep (250);

   KS_Elapse (&et);
   for (i = 0; i < 500; i++) KS_DequeueW (DEMOQX4, text, 4);
   et = KS_Elapse (&et);
   et *= ticktime;
   et /= 500; /* convert to us */
   printf (FORMAT, "dequeue 4 words", et);
   KS_Sleep (250);

   KS_Start (DEQUTASK);
   KS_Sleep (250);

   KS_Elapse (&et);
   for (i = 0; i < 1000; i++) KS_EnqueueW (DEMOQX1, text, 1);
   et = KS_Elapse (&et);
   et *= ticktime;
   et /= 1000;
   printf (FORMAT, "enqueue 1 word to a waiting higher priority task", et);
   KS_Sleep (250);

   KS_Elapse (&et);
   for (i = 0; i < 1000; i++) KS_EnqueueW (DEMOQX4, text ,4);
   et = KS_Elapse (&et);
   et *= ticktime;
   et /= 1000; /* convert to us */
   printf (FORMAT, "enqueue 4 words to a waiting higher priority task", et);
   KS_Sleep (250);


   KS_Elapse (&et);
   for (i = 0; i < 1000; i++) KS_Signal (SEM1);
   et = KS_Elapse (&et);
   et *= ticktime;
   et /= 1000; /* convert to us */
   printf (FORMAT, "signal semaphore", et);
   KS_Sleep (250);

   KS_ResetSema (SEM1);
   KS_Start (WAITTASK);
   KS_Sleep (250);

   KS_Elapse (&et);
   for (i = 0; i < 1000; i++) KS_Signal (SEM1);
   et = KS_Elapse (&et);
   et *= ticktime;
   et /= 1000; /* convert to us */
   printf (FORMAT, "signal to waiting high pri task", et);
   KS_Sleep (250);

   KS_Elapse (&et);
   for (i = 0; i < 1000; i++) KS_Signal (SEM1);
   et = KS_Elapse (&et);
   et *= ticktime;
   et /= 1000; /* convert to us */
   printf (FORMAT, "signal to waiting high pri task, with timeout", et);
   KS_Sleep (250);

   KS_Elapse (&et);
   for (i = 0; i < 1000; i++) KS_Signal (SEM2);
   et = KS_Elapse (&et);
   et *= ticktime;
   et /= 1000;
   printf (FORMAT, "signal to waitm (2)", et);
   KS_Sleep (250);

   KS_Elapse (&et);
   for (i = 0; i < 1000; i++) KS_Signal (SEM2);
   et = KS_Elapse (&et);
   et *= ticktime;
   et /= 1000;
   printf (FORMAT, "signal to waitm (2), with timeout", et);
   KS_Sleep (250);

   KS_Elapse (&et);
   for (i = 0; i < 1000; i++) KS_Signal (SEM3);
   et = KS_Elapse (&et);
   et *= ticktime;
   et /= 1000;
   printf (FORMAT, "signal to waitm (3)", et);
   KS_Sleep (250);

   KS_Elapse (&et);
   for (i = 0; i < 1000; i++) KS_Signal (SEM3);
   et = KS_Elapse (&et);
   et *= ticktime;
   et /= 1000;
   printf (FORMAT, "signal to waitm (3), with timeout", et);
   KS_Sleep (250);

   KS_Elapse (&et);
   for (i = 0; i < 1000; i++) KS_Signal (SEM4);
   et = KS_Elapse (&et);
   et *= ticktime;
   et /= 1000;
   printf (FORMAT, "signal to waitm (4)", et);
   KS_Sleep (250);

   KS_Elapse (&et);
   for (i = 0; i < 1000; i++) KS_Signal (SEM4);
   et = KS_Elapse (&et);
   et *= ticktime;
   et /= 1000;
   printf (FORMAT, "signal to waitm (4), with timeout", et);
   KS_Sleep (250);


   KS_Elapse (&et);
   for (i = 0; i < 5000; i++)
     {
     KS_LockW (DEMORES);
     KS_Unlock (DEMORES);
     }
   et = KS_Elapse (&et);
   et *= ticktime;
   et /= 10000;
   printf (FORMAT, "average lock and unlock resource", et);
   KS_Sleep (250);

   KS_Elapse (&et);
   for (i = 0; i < 5000; i++)
     {
     KS_AllocW (MAP1, &p);
     KS_Dealloc (MAP1, &p);
     }
   et = KS_Elapse (&et);
   et *= ticktime;
   et /= 10000;
   printf (FORMAT, "average alloc and dealloc memory page", et);
   KS_Sleep (250);

   putchar ('\n');
   }



 void master (void)
   {
   puts ("Demo can be terminated with ctl-C\n");
   while (1) benchmark ();
   }

