#include "iface.h"
#include "grint.h"
#include "node1.h"
#include "defines.h"
#include <math.h>


 typedef struct
   {
   double  x;
   double  y;
   } XYPOS;


 typedef struct
   {
   XYPOS  allpos [8];
   double  f1;
   double  f2;
   } PARMS;


 void ball (int id, double x, double y, double vx, double vy)
   {
   int        i, color, loops;
   FILLSTYLE  F;
   K_MSG      M;
   PARMS      parms;
   XYPOS      mypos;
   double      fx, fy, m;
   int colors [9] =
     {
     BLACK,
     LIGHTGREEN,
     LIGHTRED,
     LIGHTMAGENTA,
     YELLOW,
     LIGHTBLUE,
     GREEN,
     LIGHTCYAN,
     MAGENTA
     };

   color = colors [id + 1];

   while (1)
     {
     /* Report current position to master task */

     mypos.x = x;
     mypos.y = y;
     M.size = sizeof (XYPOS);
     M.info = id;
     M.tx_data = &mypos;
     M.rx_task = MASTER;
     KS_SendW (REPORT, 1, &M);

     /* Wait for message from master, giving current
        positions for all balls and parameters */

     M.size = sizeof (PARMS);
     M.rx_data = &parms;
     M.tx_task = MASTER;
     KS_ReceiveW (GETPOS, &M);

     /* Plot ball at current position */

     KS_LockW (GRAPHRES);
     F.type  = SOLID_FILL;
     F.color = color;
     setfillstyle (&F);
     setcolor (BLACK);
     fillellipse ((int) x + VIEW_X2 - 174, (int) y + VIEW_Y1 + 174, 4, 4);
     KS_Unlock (GRAPHRES);

     /* Compute mutual forces */

     fx = fy = 0.0;
     for (i = 0; i < 8; i++)
       {
       double dx, dy;
       if (i == id) continue;
       dx = parms.allpos [i].x - x;
       dy = parms.allpos [i].y - y;
       m = dx * dx + dy * dy + 1;
       fx += dx / m;
       fy += dy / m;
       KS_Nop();
       }
     fx *= parms.f1;
     fy *= parms.f1; 

     /* Add 'magnetic field' force */

     fx += parms.f2 * vy;
     fy -= parms.f2 * vx; 

     /* Calculate new speed vector */

     vx += fx;
     vy += fy; 

     /* Limit speed to 30.0 */

     m = sqrt (vx * vx + vy * vy);
     KS_Nop();
     if (m > 30.0)
       {
       m = 30.0 / m;
       KS_Nop(); 
       vx *= m;
       vy *= m;
       } 

     /* Calculate new position */

     x += vx;
     y += vy; 

     /* Bounce ball on screen limits */

     do
       {
       i = 0;
       if (x < -160.0) { x = -320.0 - x; vx = -vx; i++; }
       if (x  > 160.0) { x =  320.0 - x; vx = -vx; i++; }
       if (y < -160.0) { y = -320.0 - y; vy = -vy; i++; }
       if (y >  160.0) { y =  320.0 - y; vy = -vy; i++; }
       }
     while (i); 
     } 
   }



