#include <math.h>
#include "_stdio.h"
#include "iface.h"
#include "grint.h"
#include "node1.h"
#include "defines.h"


/* ------------- For TORNADO-C31 ONLY -------------------- */

#define TISRNUM 8
#define CLOCKFREQ 10.000E6

 extern void tisr_init (int unit);
 extern int  tisr_read (void);
 extern void tisr1 (void);
 extern void tisr2 (void);

#define DIVISR0    1
#define DIVTASK   10

/* ------------------------------------------------------ */


 extern int *arraybase;

 static int    count [COUNTSIZE];
 static int    testnr   = TESTISR0;
 static int    div      = DIVISR0;
 static int    refresh  = 0;
 static int    fft      = 0;
 static int    balls    = 0;
 static int    wlmon    = 1;
 static int    saturate = 0;
 static double timeunit;


 static void banners (void)
   {
   FILLSTYLE F;

   F.color = LIGHTBLUE;
   F.type  = SOLID_FILL;
   setfillstyle (&F);
   bar (0, 0, 640, 480);

   F.color = BLACK;
   setfillstyle (&F);
   bar (VIEW_X1, VIEW_Y1, VIEW_X2, VIEW_Y2);
   settextstyle (0, 0, 2);

   setcolor (DARKGRAY);
   outtextxy (34, 10, "Virtuoso Micro on TMS320C31");
   outtextxy (7, 452, "Interrupt Latencies To Different Levels");

   setcolor (YELLOW);
   outtextxy (36, 11, "Virtuoso Micro on TMS320C31");

   setcolor (WHITE);
   outtextxy (9, 453, "Interrupt Latencies To Different Levels");
   settextstyle (0, 0, 1);
   }


 static void legend (void)
   {
   FILLSTYLE F;

   F.color = BLACK;
   F.type  = SOLID_FILL;
   setfillstyle (&F);
   bar (LEGEND_X1, LEGEND_Y1, LEGEND_X2, LEGEND_Y2);

   setcolor (WHITE);
   rectangle (LEGEND_X1, LEGEND_Y1, LEGEND_X2, LEGEND_Y2);
   moveto (LEGEND_X1, LEGEND_Y1 + 24);
   lineto (LEGEND_X2, LEGEND_Y1 + 24);

   settextstyle (0, 0, 2);
   outtextxy (LEGEND_X1 + 5, LEGEND_Y1 + 5, "Keys");
   settextstyle (0, 0, 1);
   outtextxy (LEGEND_X1 + 5, LEGEND_Y1 + 30, "Levels :");

   setcolor (LIGHTGREEN);
   outtextxy (LEGEND_X1 +  5, LEGEND_Y1 +  42, "0");
   outtextxy (LEGEND_X1 +  5, LEGEND_Y1 +  54, "m");
   outtextxy (LEGEND_X1 +  5, LEGEND_Y1 + 110, "w");
   outtextxy (LEGEND_X1 +  5, LEGEND_Y1 + 122, "s");
   outtextxy (LEGEND_X1 +  5, LEGEND_Y1 + 134, "b");
   outtextxy (LEGEND_X1 +  5, LEGEND_Y1 + 146, "f");
   outtextxy (LEGEND_X1 +  5, LEGEND_Y2 -  24, "r");

   setcolor (WHITE);
   outtextxy (LEGEND_X1 + 17, LEGEND_Y1 +  42, ": ISR0");
   outtextxy (LEGEND_X1 + 17, LEGEND_Y1 +  54, ": Micro");
   outtextxy (LEGEND_X1 +  5, LEGEND_Y1 +  98, "Toggles :");
   outtextxy (LEGEND_X1 + 17, LEGEND_Y1 + 110, ": Workload");
   outtextxy (LEGEND_X1 + 17, LEGEND_Y1 + 122, ": Saturate");
   outtextxy (LEGEND_X1 + 17, LEGEND_Y1 + 134, ": Balls");
   outtextxy (LEGEND_X1 + 17, LEGEND_Y1 + 146, ": FFT");
   outtextxy (LEGEND_X1 + 17, LEGEND_Y2 -  24, ": Results");
   outtextxy (LEGEND_X1 + 33, LEGEND_Y2 -  12, "to File");
   }


 static void plotxscale (void)
   {
   int        i, x, mindiv, majdiv;
   double     scale;
   char      *text, s [10];
   FILLSTYLE  F;

   F.type  = SOLID_FILL;
   F.color = BLACK;
   setfillstyle (&F);
   bar (VIEW_X1, VIEW_Y1, VIEW_X2, VIEW_Y2);

   setcolor (WHITE);
   rectangle (VIEW_X1, VIEW_Y1, VIEW_X2, VIEW_Y2);

   timeunit = (double) div / CLOCKFREQ;
   switch (testnr)
     {
     case TESTISR0:
       mindiv = 1;
       majdiv = 5;
       text = "Latency to 8th instruction of timer isr level 0  (us)";
       break;

     case TESTTASK:
       mindiv = 10;
       majdiv = 50;
       text = "Latency to microkernel task using KS_EventW ()  (us)";
       break;
     }

   setcolor (LIGHTGREEN);
   moveto (BASE_X, BASE_Y);
   lineto (BASE_X + SIZE_X, BASE_Y);

   timeunit *= 1E6;
   scale = SIZE_X / (COUNTSIZE * timeunit);

   for (i = x = 0; x < SIZE_X; i += mindiv, x = scale * i)
     {
     moveto (BASE_X + x, BASE_Y);
     linerel (0, 4);
     }

   for (i = x = 0; x < SIZE_X; i += majdiv, x = scale * i)
     {
     moveto (BASE_X + x, BASE_Y);
     linerel (0, 8);
     moverel (-3, 2);
     if (x < SIZE_X - 30)
       {
       sprintf (s, "%d", i);
       outtext (s);
       }
     }

   setcolor (WHITE);
   outtextxy (BASE_X, BASE_Y + 20, text);
   }


 static void writefile (void)
   {
   FILE *F;
   int i, n, s, t;

   for (i = t = 0; i < COUNTSIZE; i++) t += count [i];
   if (t < 100) return;

   switch (testnr)
     {
     case TESTISR0:
       if ((F = fopen ("latisr0.txt", "wt")) == NULL) return;
       break;
     case TESTTASK:
       if ((F = fopen ("lattask.txt", "wt")) == NULL) return;
       break;
     }

   fprintf (F, "Total number of interrupts : %d\n", t);
   fputs ("    us         n         %        +%\n", F);
   fputs ("------------------------------------\n", F);

   for (i = s = 0; s < t; i++)
     {
     n = count [i];
     s += n;
     fprintf (F, "%6.2lf %9d %9.3lf %9.3lf\n",
              i * timeunit, n, 100.0 * n / t, 100.0 * s / t);
     }
   fclose (F);
   }


 static void clearcount (void)
   {
   int i;
   for (i = 0 ; i < COUNTSIZE ; i++) count [i] = 0;
   }


 void isrtest (void)
   {
   K_TIMER *Tim;
   int      i, k, x, y, t;

   clearcount ();

   KS_LockW (GRAPHRES);
   initgraph (VGA, VGAHI);
   banners ();
   legend ();
   plotxscale ();
   graphresult ();
   KS_Unlock (GRAPHRES);

   tisr_init (8765);
   KS_StartTimer (Tim = KS_AllocTimer (), 60000, 60000, RF_TIME);

   while (1)
     {

     KS_LockW (GRAPHRES);
     setcolor (WHITE);
     for (x = BASE_X, i = 0; i < COUNTSIZE; x += 2, i++)
       {
       if (count [i])
	 {
         y = BASE_Y - count [i];
         moveto (x, BASE_Y - 1);
         lineto (x, y < (BASE_Y - SIZE_Y) ? (BASE_Y - SIZE_Y) : y);
	 }
       }
     graphresult ();
     KS_Unlock (GRAPHRES);

     if (KS_Test (RF_TIME) == RC_OK) refresh = 1;

     if (k = server_pollkey ())
       {
       switch(k)
         {
 	 case 'r':
	   writefile();
	   break;

	 case 's':
	   if (saturate ^= 1) KS_Resume (DUMMY);
	   else               KS_Suspend (DUMMY);
     	   break;

	 case 'b':
	   balls ^= 1;
	   if (fft)
	     {
	     fft ^= 1;
	     KS_Signal (FFT_STOP);
	     KS_Wait (STOP_ACK);
	     }
	   KS_Signal (BLS_STOP);
	   if (! balls) KS_Wait (STOP_ACK);
	   break;

         case 'f':
	   fft ^= 1;
	   if (balls)
	     {
	     balls ^= 1;
	     KS_Signal (BLS_STOP);
	     KS_Wait (STOP_ACK);
	     }
	   KS_Signal (FFT_STOP);
	   if (! fft) KS_Wait (STOP_ACK);
	   break;

	 case 'w':
	   wlmon ^= 1;
	   if (wlmon) KS_Signal (WLM_STOP);
	   else
	     {
	     KS_Signal (WLM_STOP);
	     KS_Wait (STOP_ACK);
	     }
	   break;

	 case '0':
	   testnr = TESTISR0;
           div    = DIVISR0;
 	   break;

	 case 'm':
	   testnr = TESTTASK;
           div    = DIVTASK;
   	   break;

  	 }
       refresh = 1;
       }

     if (refresh)
       {
       KS_RestartTimer (Tim, 120000, 120000);
       refresh = 0;
       KS_LockW(GRAPHRES);
       plotxscale ();
       clearcount ();
       graphresult ();
       KS_Unlock(GRAPHRES);
       }

     switch (testnr)
       {
       case TESTISR0:
         KS_EnableISR (TISRNUM, tisr1);
	 arraybase = count;
	 KS_Sleep(30);
	 arraybase = NULL;
	 break;

       case TESTTASK:
         KS_EnableISR (TISRNUM, tisr2);
         KS_EventW (TISRNUM);
	 for (i = 0; i < 30; i++)
	   {
	   KS_EventW (TISRNUM);
	   t = tisr_read () / DIVTASK;
	   if (t >= COUNTSIZE) t = COUNTSIZE - 1;
	   count [t]++;
#ifdef DEBUG
           if (t  > 64)
             {
             KS_Signal (DEBUGGO);
             KS_EventW (TISRNUM);
             }
#endif
	   }
	 break;
       }
     }
   }


 void dummy (void)
   {
   double x, y = 1.0;
   KS_Suspend (KS_TaskId);
   while(1)
     {
     x = sin (y);
     y = cos (x);
     }
   }

