
#include "_stdio.h"
#include "iface.h"
#include "grint.h"
#include "node1.h"


 static FILLSTYLE fillpars;
 static MODEPARS modepars;
 static VIEWPORT viewport;
 static PALETTE palette;




 static void textscreen ()
   {
   KS_LockW (GRAPHRES);
   closegraph ();
   graphresult (); /* wait until closegraph is executed */
   KS_Unlock (GRAPHRES);
   }


 static void vgascreen ()
   {
   int driv, mode;

   KS_LockW (GRAPHRES);
   detectgraph (&driv, &mode);
   if (driv != VGA)
     {
     printl ("-- Sorry, I need a VGA screen !\n", CONRES, CONOQ);
     KS_Sleep (500);
     server_terminate ();
     }
   initgraph (driv, mode);
   KS_Unlock (GRAPHRES);
   }


 static int PSEG [] =
   {
    0,   0,
   -5,   5,
    0,  10,
    5,   5 };

 static int HSEG [] =
   {
    0,   0,
    4,  -4,
   60,  -4,
   64,   0,
   60,   4,
    4,   4 };

 static int VSEG [] =
   {
    0,   0,
   -4,  -4,
    0, -60,
    4, -64,
    8, -60,
    4,  -4 };

 static int SPOS [7][2] =
   {
    0,    0,
    4,  -66,
    8, -132,
   -1,   -1,
   65,   -1,
    3,  -67,
   69,  -67 };

 static int S_HSEG [] =
   {
    0,   0,
    2,  -2,
   29,  -2,
   31,   0,
   29,   2,
    2,   2 };

 static int S_VSEG [] =
   {
    0,   0,
   -2,  -2,
    0, -29,
    2, -31,
    4, -29,
    2,  -2 };

 static int S_SPOS [7][2] =
   {
    0,    0,
    2,  -33,
    4,  -66,
   -1,   -1,
   32,   -1,
    1,  -34,
   34,  -34 };

 static int SMAP [] =
   {
   0x7D, 0x50, 0x4F, 0x57, 0x72, 0x37, 0x3F, 0x54, 0x7F, 0x77
   };



 static void draw_point (int xpos, int ypos, int color)
   {
   KS_LockW (GRAPHRES);
   fillpars.type = SOLID_FILL;
   fillpars.color = color;
   setfillstyle (&fillpars);
   setcolor (color);

   viewport.left   = xpos;
   viewport.right  = xpos + 10;
   viewport.top    = ypos - 10;
   viewport.bottom = ypos;
   viewport.clip   = 0;
   setviewport (&viewport);

   fillpoly (4, PSEG);

   KS_Unlock (GRAPHRES);
   }


 static void update (int *old, int *new, int xpos, int ypos, int color,
                     int size)
   {
   int i, m, upd;

   upd = *old ^ *new;
   KS_LockW (GRAPHRES);
   for (i = 0, m = 1; i < 7; i++, m <<= 1)
     {
     if (upd & m)
       {
       fillpars.type = SOLID_FILL;
       fillpars.color = (*new & m) ? color : DARKGRAY;
       setfillstyle (&fillpars);
       setcolor ((*new & m) ? color : DARKGRAY);

       if (size)
         {
         viewport.left   = xpos + SPOS [i][0];
         viewport.right  = xpos + SPOS [i][0] + 10;
         viewport.top    = ypos + SPOS [i][1] - 10;
         viewport.bottom = ypos + SPOS [i][1];
         viewport.clip   = 0;
         setviewport (&viewport);

         if (i < 3) fillpoly (6, HSEG);
         else       fillpoly (6, VSEG);
         }
       else
         {
         viewport.left   = xpos + S_SPOS [i][0];
         viewport.right  = xpos + S_SPOS [i][0] + 10;
         viewport.top    = ypos + S_SPOS [i][1] - 10;
         viewport.bottom = ypos + S_SPOS [i][1];
         viewport.clip   = 0;
         setviewport (&viewport);

         if (i < 3) fillpoly (6, S_HSEG);
         else       fillpoly (6, S_VSEG);
         }
       }
     }
   KS_Unlock (GRAPHRES);
   *old = *new;
   }


 static void digit (int xpos,
		    int ypos,
		    int color,
		    K_SEMA in,
		    K_SEMA out,
		    int  modulus,
          int  offset,
          int  size)
   {
   int cnt, old, new;

   old = 0x7F;
   new = 0x00;
   update (&old, &new, xpos, ypos, color, size);
   cnt = 0;

   while (1)
     {
     new = SMAP [cnt];
     update (&old, &new, xpos, ypos, color, size);
     KS_Wait (in);

     if (in == SM3) KS_Signal (SM2);

     cnt += offset;
     if (cnt == modulus)
       {
       cnt = 0;
       KS_Signal (out);
       }
     }
   }


/* ------ TASKS ------------------------------------------------- */



 void digit1 (void)
   {
   digit (590, 300, LIGHTBLUE, SM1, SM3, 10, 1, 0);
   }

 void digit2 (void)
   {
   digit (485, 300, YELLOW,    SM3, SM4, 10, 1, 1);
   }

 void digit3 (void)
   {
   digit (395, 300, YELLOW,    SM4, SM5,  6, 1, 1);
   }

 void digit4 (void)
   {
   digit (290, 300, GREEN,     SM5, SM6, 10, 1, 1);
   }

 void digit5 (void)
   {
   digit (200, 300, GREEN,     SM6, SM7,  6, 1, 1);
   }

 void digit6 (void)
   {
   digit ( 95, 300,  CYAN,     SM7, SM8, 10, 1, 1);
   }

 void digit7 (void)
   {
   digit (  5, 300,  CYAN,     SM8, SM9, 10, 1, 1);
   }

 void colons (void)
   {
   while (1)
     {
     KS_Wait (SM2);
     draw_point (183, 261, LIGHTBLUE);
     draw_point (187, 197, LIGHTBLUE);
     draw_point (378, 261, LIGHTBLUE);
     draw_point (382, 197, LIGHTBLUE);
     KS_Sleep (500);
     draw_point (183, 261, DARKGRAY);
     draw_point (187, 197, DARKGRAY);
     draw_point (378, 261, DARKGRAY);
     draw_point (382, 197, DARKGRAY);
     }
   }

 void master (void)
   {
   K_TIMER *Tim;

   vgascreen ();
   KS_StartG (SEG);

   Tim = KS_AllocTimer ();
   KS_StartTimer (Tim, 100, 100, SM1);

   }


