/**************************************************************
	TMS320C3x DSP Definitions.
****************************************************************/

#ifndef __C3X_DEF__
#define __C3X_DEF__	1


//========= DSP on-chip registers defs  ===========
// (TI C3x/C4x C compiler v.5.10 and later only)

extern cregister volatile unsigned int IE;
extern cregister volatile unsigned int IF;
extern cregister volatile unsigned int IOF;
extern cregister volatile unsigned int ST;
//=================================================

//================ C3x Memory Areas ===============

#define C3X_ONCHIP_SADDR		0x800000
#define C3X_ONCHIP_EADDR		0x809FFF
#define C3X_ONCHIP_LEN			0x00A000

#define C3X_ONCHIP_RAM_SADDR	0x809800
#define C3X_ONCHIP_RAM_EADDR	0x809FFF
#define C3X_ONCHIP_RAM_LEN		0x000800
#define C32_ONCHIP_RAM_SADDR	0x87fe00
#define C32_ONCHIP_RAM_EADDR	0x87ffff
#define C32_ONCHIP_RAM_LEN		0x000200
#define C33_ONCHIP_RAM_SADDR	0x800000
#define C33_ONCHIP_RAM_EADDR	0x807FFF
#define C33_ONCHIP_RAM_LEN		0x008000
#define C3X_ONCHIP_RG_SADDR		0x808000
#define C3X_ONCHIP_RG_EADDR		0x8097FF
#define C3X_ONCHIP_RG_LEN		0x001800

#define C3X_PBUS_AREA1_SADDR	0x000000
#define C3X_PBUS_AREA1_EADDR	0x7fffff
#define C3X_PBUS_AREA1_LEN		0x800000
#define C3X_PBUS_AREA2_SADDR	0x80a000
#define C3X_PBUS_AREA2_EADDR	0xffffff
#define C3X_PBUS_AREA2_LEN		0x7f5000
#define C3X_XBUS_MSTB_SADDR		0x800000
#define C3X_XBUS_MSTB_EADDR		0x801fff
#define C3X_XBUS_MSTB_LEN		0x002000
#define C3X_XBUS_IOSTB_SADDR	0x804000
#define C3X_XBUS_IOSTB_EADDR	0x805fff
#define C3X_XBUS_IOSTB_LEN		0x002000


/************  C3x Peripheral Registers *****************/

/*-------------- SIO ports ---------------------*/
int *C3X_SIO0_GC		= (int *)0x808040;
int *C3X_SIO0_XC		= (int *)0x808042;
int *C3X_SIO0_RC		= (int *)0x808043;
int *C3X_SIO0_RXTC		= (int *)0x808044;
int *C3X_SIO0_RXTCNT	= (int *)0x808045;
int *C3X_SIO0_RXTPER	= (int *)0x808046;
int *C3X_SIO0_XD		= (int *)0x808048;
int *C3X_SIO0_RD		= (int *)0x80804C;

int *C3X_SIO1_GC		= (int *)0x808050;
int *C3X_SIO1_XC		= (int *)0x808052;
int *C3X_SIO1_RC		= (int *)0x808053;
int *C3X_SIO1_RXTC		= (int *)0x808054;
int *C3X_SIO1_RXTCNT	= (int *)0x808055;
int *C3X_SIO1_RXTPER	= (int *)0x808056;
int *C3X_SIO1_XD		= (int *)0x808058;
int *C3X_SIO1_RD		= (int *)0x80805C;

/*-------- Timers --------*/
int *C3X_TM1_GC			= (int *)0x808030;
int *C3X_TM1_CNT		= (int *)0x808034;
int *C3X_TM1_PER		= (int *)0x808038;
int *C3X_TM0_GC			= (int *)0x808020;
int *C3X_TM0_CNT		= (int *)0x808024;
int *C3X_TM0_PER		= (int *)0x808028;

/*-------- DMA --------*/
int *C3X_DMA0_GC		= (int *)0x808000;
int *C3X_DMA0_SA		= (int *)0x808004;
int *C3X_DMA0_DA		= (int *)0x808006;
int *C3X_DMA0_TC		= (int *)0x808008;

/*-------- P-BUS/X-BUS control registers --------*/
int *C3X_PBUS_GC		= (int *)0x808064;
int *C3X_XBUS_GC		= (int *)0x808060;
int *C32_STB0_GC		= (int *)0x808064;
int *C32_STB1_GC		= (int *)0x808068;
int *C32_IOSTB_GC		= (int *)0x808060;

/************  C3x macros & IRQ control *****************/

#define C3X_INT_ENABLE				asm("	or    2000h,st")
#define C3X_INT_DISABLE				asm("	and  ~2000h,st")
#define C3X_INT_LEVEL				asm("	and  ~4000h,st")	// C32 only
#define C3X_INT_EDGE				asm("	or    4000h,st")	// C32 only

#define C3X_CACHE_ENABLE		    {asm("	and  ~(800h | 400h),st");asm("	or    (1000h | 800h),st");}
#define	C3X_CACHE_DISABLE			asm("	and  ~800h,st")	
#define	C3X_CACHE_FREEZE			asm("	or    400,st")
#define	C3X_CACHE_UNFREEZE			asm("	and  ~400,st")

#define	C3X_INT_DISABLE_ALL			IE=0
#define	C3X_INT_CLEAR_ALL			IF=0

#define	C3X_XINT_ENABLE(i)		 	asm("   or   (1<<" #i "),ie")
#define	C3X_XINT_DISABLE(i)			asm("	and ~(1<<" #i "),ie")
#define	C3X_XINT_CLEAR(i)			asm("	and ~(1<<" #i "),if")
#define	C3X_XINT_SET(i)				asm("	or   (1<<" #i "),if")
#define	C3X_IE_XINT_DATA(i)			(IE & (1<< (i&0x3)))
#define	C3X_IF_XINT_DATA(i)			(IF & (1<< (i&0x3)))


#define	C3X_SX0_INT_ENABLE			asm("	or   10h,ie")
#define	C3X_SX0_INT_DISABLE			asm("	and ~10h,ie")
#define	C3X_SX0_INT_CLEAR			asm("	and ~10h,if")
#define	C3X_SX0_INT_SET				asm("	or   10h,if")
#define	C3X_IE_SX0_INT_DATA			(IE & 0x10)
#define	C3X_IF_SX0_INT_DATA			(IF & 0x10)

#define	C3X_SR0_INT_ENABLE			asm("	or   20h,ie")
#define	C3X_SR0_INT_DISABLE			asm("	and ~20h,ie")
#define	C3X_SR0_INT_CLEAR			asm("	and ~20h,if")
#define	C3X_SR0_INT_SET				asm("	or   20h,if")
#define	C3X_IE_SR0_INT_DATA			(IE & 0x20)
#define	C3X_IF_SR0_INT_DATA			(IF & 0x20)

#define	C3X_SX1_INT_ENABLE			asm("	or   40h,ie")
#define	C3X_SX1_INT_DISABLE			asm("	and ~40h,ie")
#define	C3X_SX1_INT_CLEAR			asm("	and ~40h,if")
#define	C3X_SX1_INT_SET				asm("	or   40h,if")
#define	C3X_IE_SX1_INT_DATA			(IE & 0x40)
#define	C3X_IF_SX1_INT_DATA			(IF & 0x40)

#define	C3X_SR1_INT_ENABLE			asm("	or   80h,ie")
#define	C3X_SR1_INT_DISABLE			asm("	and ~80h,ie")
#define	C3X_SR1_INT_CLEAR			asm("	and ~80h,if")
#define	C3X_SR1_INT_SET				asm("	or   80h,if")
#define	C3X_IE_SR1_INT_DATA			(IE & 0x80)
#define	C3X_IF_SR1_INT_DATA			(IF & 0x80)

#define	C3X_TM0_INT_ENABLE			asm("	or   100h,ie")
#define	C3X_TM0_INT_DISABLE			asm("	and ~100h,ie")
#define	C3X_TM0_INT_SET				asm("	or   100h,if")
#define	C3X_TM0_INT_CLEAR			asm("	and ~100h,if")
#define	C3X_IE_TM0_INT_DATA			(IE & 0x100)
#define	C3X_IF_TM0_INT_DATA			(IF & 0x100)

#define	C3X_TM1_INT_ENABLE			asm("	or   200h,ie")
#define	C3X_TM1_INT_DISABLE			asm("	and ~200h,ie")
#define	C3X_TM1_INT_SET				asm("	or   200h,if")
#define	C3X_TM1_INT_CLEAR			asm("	and ~200h,if")
#define	C3X_IE_TM1_INT_DATA			(IE & 0x200)
#define	C3X_IF_TM1_INT_DATA			(IF & 0x200)

#define	C3X_DMA0_INT_ENABLE			asm("	or   400h,ie")
#define	C3X_DMA0_INT_DISABLE		asm("	and ~400h,ie")
#define	C3X_DMA0_INT_SET			asm("	or   400h,if")
#define	C3X_DMA0_INT_CLEAR			asm("	and ~400h,if")
#define	C3X_IE_DMA0_INT_DATA		(IE & 0x400)
#define	C3X_IF_DMA0_INT_DATA		(IF & 0x400)


/*------------ DMA controller interrupts -----------------*/
#define	C3X_DMA0_DMA_XINT0_ENABLE	asm("	or   10000h,ie")
#define	C3X_DMA0_DMA_XINT0_DISABLE	asm("	and ~10000h,ie")
#define	C3X_DMA0_DMA_XINT1_ENABLE	asm("	or   20000h,ie")
#define	C3X_DMA0_DMA_XINT1_DISABLE	asm("	and ~20000h,ie")
#define	C3X_DMA0_DMA_XINT2_ENABLE	asm("	or   40000h,ie")
#define	C3X_DMA0_DMA_XINT2_DISABLE	asm("	and ~40000h,ie")
#define	C3X_DMA0_DMA_XINT3_ENABLE	asm("	or   80000h,ie")
#define	C3X_DMA0_DMA_XINT3_DISABLE	asm("	and ~80000h,ie")
#define	C3X_DMA0_DMA_SX0_ENABLE		asm("	or   100000h,ie")
#define	C3X_DMA0_DMA_SX0_DISABLE	asm("	and ~100000h,ie")
#define	C3X_DMA0_DMA_SR0_ENABLE		asm("	or   200000h,ie")
#define	C3X_DMA0_DMA_SR0_DISABLE	asm("	and ~200000h,ie")
#define	C3X_DMA0_DMA_SX1_ENABLE		asm("	or   400000h,ie")
#define	C3X_DMA0_DMA_SX1_DISABLE	asm("	and ~400000h,ie")
#define	C3X_DMA0_DMA_SR1_ENABLE		asm("	or   800000h,ie")
#define	C3X_DMA0_DMA_SR1_DISABLE	asm("	and ~800000h,ie")
#define	C3X_DMA0_DMA_TM0_ENABLE		asm("	or   1000000h,ie")
#define	C3X_DMA0_DMA_TM0_DISABLE	asm("	and ~1000000h,ie")
#define	C3X_DMA0_DMA_TM1_ENABLE		asm("	or   2000000h,ie")
#define	C3X_DMA0_DMA_TM1_DISABLE	asm("	and ~2000000h,ie")
#define	C3X_DMA0_DMA_ENABLE			asm("	or   4000000h,ie")
#define	C3X_DMA0_DMA_DISABLE		asm("	and ~4000000h,ie")

#endif
