/***************************************************************
  TORNADO-3x utilities (C3x DSP Environment).
  (C) MicroLAB Systems Ltd, 1993-2000

  covers:
  	T31  rev.1..4A
	T31Z rev.1..2B
	T31M rev.1A
	T33  rev.1A
	T32L rev.1B
****************************************************************/


#ifndef __T3X_DEF__
#define __T3X_DEF__	1


#define	ON			1
#define	OFF			0

//================ T3x Memory/IO Areas ===============

	//----------- SRAM
#define	T3X_SRAM_SADDR		0x000000

#define	T31_SRAM_EADDR		0x1fffff	// for T31 rev. below 4A
#define	T31_REV4_SRAM_EADDR	0x0fffff	// specific to T31 rev.4A
#define	T31Z_SRAM_EADDR		0x0fffff
#define	T31M_SRAM_EADDR		0x0fffff
#define	T33_SRAM_EADDR		0x0fffff	
#define	T32L_SRAM_EADDR		0x07ffff	// T32L only
#define	T31_SRAM_LEN		0x200000	// for T31 rev. below 4A
#define	T31_REV4_SRAM_LEN	0x100000        // specific to T31 rev.4A
#define	T31Z_SRAM_LEN		0x100000
#define	T31M_SRAM_LEN		0x100000
#define	T33_SRAM_LEN		0x100000
#define	T32L_SRAM_LEN		0x080000

#define	T3X_SRAM_EADDR_MAX	0x1fffff
#define	T3X_SRAM_LEN_MAX	0x200000

	//----------- PIOX area
#define T31_PIOX_SADDR		0xC00000	// for T31 rev. below 4A
#define T31_PIOX_EADDR		0xffffff
#define T31_PIOX_LEN		0x400000
#define T31_REV4_PIOX_SADDR	0xF00000	// for T31 rev.4A
#define T31_REV4_PIOX_EADDR	0xffffff
#define T31_REV4_PIOX_LEN	0x100000
#define T31_PIOX16_SADDR	0xff0000
#define T31_PIOX16_EADDR	0xffffff
#define T31_PIOX16_LEN		0x010000

#define T33_PIOX_SADDR		0xF00000
#define T33_PIOX_EADDR		0xffffff
#define T33_PIOX_LEN		0x100000
#define T33_PIOX16_SADDR	0xff0000
#define T33_PIOX16_EADDR	0xffffff
#define T33_PIOX16_LEN		0x010000

#define T3X_PIOX_SADDR		0xF00000	// recommended universal PIOX/PIOX-16 addr range
#define T3X_PIOX_EADDR		0xffffff
#define T3X_PIOX_LEN		0x100000
#define T3X_PIOX16_SADDR	0xff0000
#define T3X_PIOX16_EADDR	0xffffff
#define T3X_PIOX16_LEN		0x010000

	//----------- PIOX/SIOX reset control rg
	// !! available for T33 only in case bit DSP_PXSX_SFT_RESET_EN
	//    in COMPAT_FRG of Host ISA-bus I/O I/F is set to '1'.
#define T3X_PXSX_RUN_ADDR	0x900000	

	//------ recommended P-BUS/X-BUS configurations ----------
#define T3X_PBUS_GC_DATA	0x800		// external ready, 64k
#define T33_PBUS_GC_DATA	0x700		// external ready, 128k
#define T3X_XBUS_GC_DATA	0x000		// external ready

#define T32L_STB0_GC_DATA	0xF0600		// T32L only
#define T32L_STB1_GC_DATA	0xF0600		// T32L only
#define T32L_IOSTB_GC_DATA	0x0			// T32L only


//================================================================


//********** Macros for MLOCK/MH_RQ Control *************

#define T3X_init_mlock()	{asm (" ldi 6,iof");}	// configure IOF register for MLOCK support

#define T3X_set_mh_rq()		{asm(" push dp");asm(" ldp 0H");asm (" iack @0H");asm(" pop dp");}
#define T3X_set_mlock()		{asm(" push dp");asm(" push r0");asm(" ldp 0");asm(" ldii @0H,r0");asm(" pop r0");asm(" pop dp");}
#define T3X_clr_mlock()		{asm(" push dp");asm(" push r0");asm(" ldp 0");asm(" ldi @0,r0");asm (" stii r0,@0H");asm(" pop r0");asm(" pop dp");}
//*******************************************************

	//--- PIOX/SIOX Reset Control
	// !! available for T33 only in case bit DSP_PXSX_SFT_RESET_EN
	//    in COMPAT_FRG of Host ISA-bus I/O I/F is set to '1'.
#define	T3X_PXSX_RUN_BITMASK	3
#define	T3X_PX_RUN_BITMASK		1
#define	T3X_PX_RESET			0
#define	T3X_PX_RUN				1
#define	T3X_SX_RUN_BITMASK		2
#define	T3X_SX_RESET			0
#define	T3X_SX_RUN				2
#define	T3X_PX_RUN_SHIFT		0
#define	T3X_SX_RUN_SHIFT		1

#define T3X_set_pxsx_run(v)	(*(unsigned *)  T3X_PXSX_RUN_ADDR)=v
#define T3X_pxsx_run()		((*(unsigned *) T3X_PXSX_RUN_ADDR) & T3X_PXSX_RUN_BITMASK)

#define T3X_set_px_run(v)	T3X_set_pxsx_run((T3X_pxsx_run() & ~T3X_PX_RUN_BITMASK) | (v & T3X_PX_RUN_BITMASK))
#define T3X_px_run()		(T3X_pxsx_run() & T3X_PX_RUN_BITMASK)
#define T3X_set_sx_run(v)	T3X_set_pxsx_run((T3X_pxsx_run() & ~T3X_SX_RUN_BITMASK) | ((v &T3X_PX_RUN_BITMASK)<<T3X_SX_RUN_SHIFT))
#define T3X_sx_run()		((T3X_pxsx_run() & T3X_SX_RUN_BITMASK)>>T3X_SX_RUN_SHIFT)

//****************************************************


#endif
