/***************************************************************************

  TORNADO-30 Control Utilities. v.2.0
  (C) MicroLAB Systems Ltd, 1994-1997

  note: this file include definitions and function declarations only
	(should be used in conjunction with T30.C file in order to
         keep definitions and vars/functions separately in a library manner)

****************************************************************************/


#include <conio.h>

typedef unsigned char BYTE ; 


#define	T30_OK				0		// error codes
#define	T30_PARAM_ERR			-1
#define	T30_ISA_IF_CNF_OFF_ERR		-2
#define	T30_SMP_OFF_ERR			-3
#define	T30_DPM_OFF_ERR			-4
#define	T30_HW_ERR			-5
#define	T30_INCOMPAT_ISA_IF_CNF_ERR	-6


//================= T30 specific definitions ==============================


#define T30_DSP_EADDR		0xffffffL  // C31 address range
#define T30_DSP_SADDR		0L
#define T30_DSP_ON_CHIP_SADDR	0x800000L  // C31 on-chip memory address range
#define T30_DSP_ON_CHIP_EADDR	0x809FFFL

#define T30_SB_EADDR		0xffffffL  // SB address range
#define T30_SB_SADDR		0L

#define T30_SRAM_EADDR		0x1fffffL  // SRAM base/end address
#define T30_SRAM_SADDR		0
#define T30_SRAM_LEN_MAX	0x200000L  // total SRAM length

#define T30_PIOX_SADDR		0xF00000L  // PIOX base/end address
#define T30_PIOX_EADDR		0xFFFFFFL
#define T30_PIOX_LEN_MAX	0x100000L


#define T30_DPM_WORD_LEN	0x4L		// DPM word len (in bytes)

#define T30_DPM_TOTAL_LEN	0x2000L		// total DPM length
#define T30_DPM_SEGM16K_LEN	0x4000L		// DPM-16K window length
#define T30_DPM_SEGM32K_LEN	0x8000L		// DPM-32K window length

#define T30_DPM_SEGM32K_SEL		0x80		
#define T30_DPM_PAGE_SEL		0x10		
#define T30_DPM_TOTAL_PAGES		0x2		
#define T30_DPM_SEGM16K_BITMASK		0xf		
#define T30_DPM_SEGM32K_BITMASK		0x7		
#define T30_DPM_SEGM32K_SEL_BITMASK	0x80		
#define T30_DPM_PAGE_SEL_BITMASK	0x10		

#define T30_DPM0_SADDR		0x0000L		// DPM Z#0 start addr
#define T30_DPM0_EADDR		0x1fefL		// DPM Z#0 end addr
#define T30_DPM0_LEN		0x1ff0L		// DPM Z#0 length
#define T30_DPM1_SADDR		0x1ff8L		// DPM Z#1 start addr
#define T30_DPM1_EADDR		0x1fffL		// DPM Z#1 end addr
#define T30_DPM1_LEN		0x8L		// DPM Z#1 length

#define T30_MH_DP_RQ_ADDR	0x1ffe		// M->H request via DPRAM
#define T30_HM_DP_RQ_ADDR	0x1fff		// H->M request via DPRAM

#define T30_DPM_SEMZ_SADDR	0x1ff0		// SEMAPHORE Zone SADDR
#define T30_DPM_SEMZ_EADDR	0x1ff7		// SEMAPHORE Zone EADDR
#define T30_DPM_SEMZ_LEN	0x8		// SEMAPHORE Zone len
#define T30_DPM_SEM_QTY		8		// qty of SEMAPHOREs

#define T30_DPM_SEM_OFF		0x1		// SEMAPHORE is OFF
#define T30_DPM_SEM_ON		0x0		// SEMAPHORE is ON
#define T30_DPM_SEM_MASK	0x1		// SEMAPHORE bitmask






//------------------- T30 ISA I/O I/F ---------------------- 

extern	unsigned	t30_io_baddr;        // T30 ISA I/O base address
extern	unsigned	T30_IO_BADDR_LIST[16];
#define	T30_IO_BADDR_CNF_QTY	16


//------------------- UECM I/F ---------------------- 

extern	unsigned	uecm_io_baddr;	// UECM I/O base address
extern	unsigned	UECM_IO_BADDR_LIST[4];

#define	UECM_IO_BADDR_CNF_QTY	4




//------------------- T30 ISA-CNF & SMP I/F ---------------------- 

#define	T30_SMP_OFF		0


#define	T30_ISA_IF_CNF_ID_MIN	0
#define	T30_ISA_IF_CNF_ID_MAX	5
#define	T30_ISA_IF_CNF_OFF	-1

extern	unsigned	t30_smp_segm;		// T30 SMP Window parameters
extern	unsigned	t30_smp_len;		// SMP length
extern	unsigned	t30_smp_qty;		// qty (total number) of SMPs
extern	int   		t30_isa_if_cnf_id;	// # (id) of ISA IF CNF


extern	unsigned	T30_SMP_LEN_CNF_LIST[6];
extern	unsigned	T30_SMP_QTY_CNF_LIST[6];

extern	unsigned	T30_SMP_SEGM_LIST[32]		;
extern	unsigned	T30_SMP_TOTAL_LEN_LIST[32];

#define	T30_SMP_SEGM_CNF_QTY		32
#define	T30_SMP_32KB_CNF_QTY		8
#define	T30_SMP_16KB_CNF_QTY		16



//------------------- T30 DPM I/F ---------------------- 

#define	T30_DPM_OFF		0


extern	unsigned	t30_dpm_segm, t30_dpm_segm_len;	// T30 DPM Window parameters
extern	unsigned	T30_DPM_SEGM16K_LIST[16],T30_DPM_SEGM32K_LIST[8];

#define	T30_DPM_SEGM16K_CNF_QTY	16
#define	T30_DPM_SEGM32K_CNF_QTY	8



//------------------- T30 I/O registers ---------------------- 

#define	T30_SMP_RG		(t30_io_baddr)
#define	T30_SMP_RG_LSB		(t30_io_baddr) 
#define	T30_SMP_RG_MSB		(t30_io_baddr+1) 
#define	T30_SMP0_RG		(t30_io_baddr)
#define	T30_SMP0_RG_LSB		(t30_io_baddr) 
#define	T30_SMP0_RG_MSB		(t30_io_baddr+1) 
#define	T30_SMP1_RG		(t30_io_baddr+0x400)
#define	T30_SMP1_RG_LSB		(t30_io_baddr+0x400) 
#define	T30_SMP1_RG_MSB		(t30_io_baddr+1+0x400) 
#define	T30_CNTR_RG		(t30_io_baddr+2)
#define	T30_FLAG_CNTR_RG	(t30_io_baddr+3)
#define	T30_FLAG_STAT_RG	(t30_io_baddr+3)
#define	T30_FLAG_SEL_RG		(t30_io_baddr+0x403)


#define T30_MRESET_RESET 	0x0		// Control Register bits
#define T30_MRESET_GO		0x1
#define T30_MRESET_UECM		0x2
#define T30_MRESET_RESET_2	0x3
#define T30_MRESET_BITMASK	0x3

#define T30_SB_GLOCK_BITMASK	0x4
#define T30_SB_LOCK_BITMASK	0x8

#define T30_SB_CCL_BYTE		0
#define T30_SB_CCL_W16		0x10
#define T30_SB_CCL_W32		0x20
#define T30_SB_CCL_W32_2	0x30
#define T30_SB_CCL_BITMASK	0x30

#define T30_SB_ERR_IE_BITMASK	0x80
#define T30_MH_RQ_IE_BITMASK	0x40


#define T30_SYS_STAT_FRG	0x0		// Flag registers
#define T30_SYS_STAT_FRG_0	0x0
#define T30_SYS_STAT_FRG_1	0x10
#define T30_SYS_STAT_FRG_2	0x20
#define T30_SYS_STAT_FRG_3	0x30

#define T30_SET_HM_RQ_FRG	0x10
#define T30_CLR_MH_RQ_FRG	0x20
#define T30_CLR_SB_ERR_FRG	0x30
#define T30_T31_FCNTR_FRG	0x00

#define T30_ISA_MI_BADDR_FRG	0xE0
#define T30_ISA_DPMI_BADDR_FRG	0xE1
#define T30_LCA_CNTR_FRG	0xEF
#define T30_LCA_CCLK_FRG	0xEE

#define T30_DEV_ID0_FRG		0xF0
#define T30_DEV_ID1_FRG		0xF1


#define T30_SB_ACK_FLAG_BITMASK		0x1		// SYS_STATUS_FRG Flag Status register bits
#define T30_SB_ERR_FLAG_BITMASK		0x80
#define T30_MH_RQ_FLAG_BITMASK		0x40
#define T30_MH_DP_RQ_FLAG_BITMASK	0x20

#define T31_CLR_MH_RQ_FLAG	0x20		// T31 Flag Sel register bits
#define T31_CLR_SB_ERR_FLAG	0x30
#define T31_HM_RQ_FLAG		0x10


#define T30_MCLOCK		20000000.00	// DSP clock freq in Hz

//-------------------------------------------------------------------------



//-------------------- macros for CONTROL REGISTER ---------------------------

#define T30_CNTR_RG_DATA	inp(T30_CNTR_RG) 

#define T30_MRESET_DATA		(inp(T30_CNTR_RG) & T30_MRESET_BITMASK) 
#define T30_SET_MRESET		outp(T30_CNTR_RG,inp(T30_CNTR_RG)&~T30_MRESET_BITMASK) 
#define T30_SET_MGO		outp(T30_CNTR_RG,inp(T30_CNTR_RG)&~T30_MRESET_BITMASK | T30_MRESET_GO) 
#define T30_SET_MRESET_UECM	outp(T30_CNTR_RG,inp(T30_CNTR_RG)&~T30_MRESET_BITMASK | T30_MRESET_UECM) 

#define T30_SB_CCL_DATA		(inp(T30_CNTR_RG) & T30_SB_CCL_BITMASK)
#define T30_SET_SB_CCL_BYTE	outp(T30_CNTR_RG,inp(T30_CNTR_RG)&~T30_SB_CCL_BITMASK |T30_SB_CCL_BYTE)
#define T30_SET_SB_CCL_W16	outp(T30_CNTR_RG,inp(T30_CNTR_RG)&~T30_SB_CCL_BITMASK |T30_SB_CCL_W16)
#define T30_SET_SB_CCL_W32	outp(T30_CNTR_RG,inp(T30_CNTR_RG)&~T30_SB_CCL_BITMASK |T30_SB_CCL_W32)

#define T30_SB_GLOCK_DATA	(inp(T30_CNTR_RG) &  T30_SB_GLOCK_BITMASK)
#define T30_SET_SB_GLOCK_ON	outp(T30_CNTR_RG,inp(T30_CNTR_RG) |  T30_SB_GLOCK_BITMASK)
#define T30_SET_SB_GLOCK_OFF	outp(T30_CNTR_RG,inp(T30_CNTR_RG) & ~T30_SB_GLOCK_BITMASK)
#define T30_SB_LOCK_DATA	(inp(T30_CNTR_RG) &  T30_SB_LOCK_BITMASK)
#define T30_SET_SB_LOCK_ON	outp(T30_CNTR_RG,inp(T30_CNTR_RG) |  T30_SB_LOCK_BITMASK)
#define T30_SET_SB_LOCK_OFF	outp(T30_CNTR_RG,inp(T30_CNTR_RG) & ~T30_SB_LOCK_BITMASK)

#define T30_SB_ERR_IE_DATA	(inp(T30_CNTR_RG) &  T30_SB_ERR_IE_BITMASK)
#define T30_SET_SB_ERR_IE_ON	outp(T30_CNTR_RG,inp(T30_CNTR_RG) |  T30_SB_ERR_IE_BITMASK)
#define T30_SET_SB_ERR_IE_OFF	outp(T30_CNTR_RG,inp(T30_CNTR_RG) & ~T30_SB_ERR_IE_BITMASK)

#define T30_MH_RQ_IE_DATA	(inp(T30_CNTR_RG) &  T30_MH_RQ_IE_BITMASK)
#define T30_SET_MH_RQ_IE_ON	outp(T30_CNTR_RG,inp(T30_CNTR_RG) |  T30_MH_RQ_IE_BITMASK)
#define T30_SET_MH_RQ_IE_OFF	outp(T30_CNTR_RG,inp(T30_CNTR_RG) & ~T30_MH_RQ_IE_BITMASK)

#define T30_INIT		{outp(T30_CNTR_RG,T30_MRESET_RESET);T30_clr_sb_err();T30_clr_mh_rq();}



//-------------------- macros for UECM-30 control ---------------------------

#define UECM_SELECT_MDSP	outp(uecm_io_baddr+0x800,0)
#define UECM_SELECT_XDSP	outp(uecm_io_baddr+0x801,0)
#define UECM_RESET		outp(uecm_io_baddr+0x818,0)



//------------------ macros for FLAGs control/status --------------------

#define T30_FLAG_SEL_RG_DATA		inp(T30_FLAG_SEL_RG)
#define T30_SET_FLAG_SEL_RG(n)		outp(T30_FLAG_SEL_RG,n)
#define	T30_SET_FLAG_SEL_RG_VERIF(n)	T30_set_flag_sel_rg_verif(n)

#define T30_FLAG_STAT_RG_DATA		inp(T30_FLAG_STAT_RG)
#define T30_SET_FLAG_CNTR_RG(n)		outp(T30_FLAG_CNTR_RG,n)

#define T30_SET_FLAG_RG_VERIF(r,v)	T30_set_flag_rg_verif(r,v)
#define T30_FLAG_RG_DATA(r)		T30_flag_rg_data(r)


#define T30_SYS_STAT_FRG_DATA	T30_sys_stat_frg_data()
#define T30_SB_ERR_DATA		T30_sb_err_data()
#define T30_SB_ACK_DATA		T30_sb_ack_data()
#define T30_MH_RQ_DATA		T30_mh_rq_data()
#define T30_MH_DP_RQ_DATA	T30_mh_dp_rq_data()


#define T30_SET_HM_RQ		T30_set_hm_rq()
#define T30_CLR_SB_ERR		T30_clr_sb_err()
#define T30_CLR_MH_RQ		T30_clr_mh_rq()


#define T31_FLAG_STATUS_RG_DATA	(t30_isa_if_cnf_id==0 ? inp(T30_FLAG_STAT_RG) & (T30_SB_ERR_FLAG_BITMASK|T30_MH_RQ_FLAG_BITMASK|T30_SB_ACK_FLAG_BITMASK|T30_MH_DP_RQ_FLAG_BITMASK):T30_ISA_IF_CNF_OFF_ERR)
#define T31_SB_ERR_DATA		(t30_isa_if_cnf_id==0 ? inp(T30_FLAG_STAT_RG) & T30_SB_ERR_FLAG_BITMASK:T30_ISA_IF_CNF_OFF_ERR)
#define T31_MH_RQ_DATA		(t30_isa_if_cnf_id==0 ? inp(T30_FLAG_STAT_RG) & T30_MH_RQ_FLAG_BITMASK :T30_ISA_IF_CNF_OFF_ERR)
#define T31_SB_ACK_DATA		(t30_isa_if_cnf_id==0 ? inp(T30_FLAG_STAT_RG) & T30_SB_ACK_FLAG_BITMASK:T30_ISA_IF_CNF_OFF_ERR)

#define T31_SET_HM_RQ		{if(t30_isa_if_cnf_id==0) {outp(T30_FLAG_SEL_RG,T30_T31_FCNTR_FRG);outp(T30_FLAG_CNTR_RG,T31_HM_RQ_FLAG);}}
#define T31_CLR_SB_ERR		{if(t30_isa_if_cnf_id==0) {outp(T30_FLAG_SEL_RG,T30_T31_FCNTR_FRG);outp(T30_FLAG_CNTR_RG,T31_CLR_SB_ERR_FLAG);}} 
#define T31_CLR_MH_RQ		{if(t30_isa_if_cnf_id==0) {outp(T30_FLAG_SEL_RG,T30_T31_FCNTR_FRG);outp(T30_FLAG_CNTR_RG,T31_CLR_MH_RQ_FLAG);}}



int	T30_set_flag_sel_rg_verif(unsigned frg);
int	T30_set_flag_rg_verif(unsigned frg,BYTE value);
BYTE	T30_flag_rg_data(unsigned frg);
BYTE	T30_sys_stat_frg_data();
int	T30_sb_err_data();
int	T30_sb_ack_data();
int	T30_mh_rq_data();
int	T30_mh_dp_rq_data();
int	T30_set_hm_rq();
int	T30_clr_sb_err();
int	T30_clr_mh_rq();

#define __MK_FP__(seg,ofs)	((void far *)(((unsigned long)(seg) << 16) | (unsigned)(ofs)))


#define T30_SET_SMP_RG(page)	outpw(T30_SMP_RG,page)
#define T30_SMP_RG_DATA		inpw(T30_SMP_RG)
#define T30_SET_SMP0_RG(page)	outpw(T30_SMP_RG,page)
#define T30_SMP0_RG_DATA	inpw(T30_SMP_RG)
#define T30_SET_SMP1_RG(page)	outpw(T30_SMP1_RG,page)
#define T30_SMP1_RG_DATA	inpw(T30_SMP1_RG)


#define T30_MAKE_SMP(saddr)		((unsigned)( (long)(saddr/t30_smp_len)))
#define T30_MAKE_SMP_PTR(saddr)		(__MK_FP__(t30_smp_segm,(unsigned)((long)(saddr%t30_smp_len))))
#define T30_MAKE_SMP0_PTR(saddr)	(__MK_FP__(t30_smp_segm,(unsigned)((long)(saddr%t30_smp_len))))
#define T30_MAKE_SMP1_PTR(saddr)	(__MK_FP__(t30_smp_segm,t30_smp_len+(unsigned)((long)(saddr%t30_smp_len))))

#define T30_SET_SMP_SEGM(s)	T30_set_smp_segm(s)
#define T30_SET_SMP_OFF		T30_set_smp_off()

int	T30_set_smp_segm(unsigned segm);
int	T30_set_smp_off();
int	T30_smp_ptr(unsigned long addr,unsigned *page,void **p,unsigned page_id);
int	T30_set_smp_ptr(unsigned long addr,void **p,unsigned page_id);

//------------------ macros/function for DPM access --------------------


#define T30_MAKE_DPM_PTR(dpmaddr)	(__MK_FP__(t30_dpm_segm,(unsigned)((long)dpmaddr) ))

#define T30_SET_DPM_SEGM(s,l)	T30_set_dpm_segm(s,l)
#define T30_SET_DPM_OFF		T30_set_dpm_off()


int	T30_get_dpm_segm_cnf();
int	T30_set_dpm_segm(unsigned segm,unsigned len);
int	T30_set_dpm_off();
int	T30_set_dpm_page(int page);
int	T30_set_dpm_ptr(unsigned long dpmaddr,void **p);
int	T30_dpm_sem_data(unsigned sem);
int	T30_set_dpm_sem(unsigned sem,unsigned value);

//------------------ macros/function for DEV_ID access --------------------

int	T30_dev_id_data(unsigned *dev_id);


//========================================================================


