/***************************************************************************

  TORNADO-31 Control Utilities. v.2.0
  (C) MicroLAB Systems Ltd, 1994-1997

  note: this file include definitions and function declarations only
	(should be used in conjunction with T31.C file in order to
         keep definitions and vars/functions separately in a library manner)

****************************************************************************/


#include <conio.h>

typedef unsigned char BYTE ; 


#define	T31_OK				0		// error codes
#define	T31_PARAM_ERR			-1
#define	T31_SMP_OFF_ERR			-3
#define	T31_HW_ERR			-5
#define	T31_NOT_T31_1A_FCNTR_FRG_ERR	-10


//================= T31 specific definitions ==============================


#define T31_DSP_EADDR		0xffffffL  // C3x address range
#define T31_DSP_SADDR		0L
#define T31_DSP_ON_CHIP_SADDR	0x800000L  // C3x on-chip memory address range
#define T31_DSP_ON_CHIP_EADDR	0x809FFFL

#define T31_SB_EADDR		0xffffffL  // SB address range
#define T31_SB_SADDR		0L

#define T31_SRAM_EADDR		0x0fffffL  // SRAM base/end address
#define T31_SRAM_SADDR		0
#define T31_SRAM_LEN_MAX	0x100000L  // total SRAM length



//------------------- T31 ISA I/O I/F ---------------------- 

extern	unsigned	t31_io_baddr;        // T31 ISA I/O base address
extern	unsigned	T31_IO_BADDR_LIST[8];
#define	T31_IO_BADDR_CNF_QTY	8


//------------------- UECM I/F ---------------------- 

extern	unsigned	uecm_io_baddr;	// UECM I/O base address
extern	unsigned	UECM_IO_BADDR_LIST[4];

#define	UECM_IO_BADDR_CNF_QTY	4




//------------------- T31 ISA-CNF & SMP I/F ---------------------- 

#define	T31_SMP_OFF		0


extern	unsigned	t31_smp_segm;		// T31 SMP Window parameters
#define	t31_smp_len	0x8000L			// SMP length
#define	T31_SMP_LEN	0x8000L			// SMP length

extern	unsigned	T31_SMP_SEGM_LIST[8]		;
#define T31_SMP_SEGM_CNF_QTY		8


//------------------- T3Z I/O registers ---------------------- 

#define	T31_SMP_RG		(t31_io_baddr)
#define	T31_SMP_RG_LSB		(t31_io_baddr) 
#define	T31_SMP_RG_MSB		(t31_io_baddr+1) 
#define	T31_CNTR_RG		(t31_io_baddr+2)
#define	T31_FLAG_CNTR_RG	(t31_io_baddr+3)
#define	T31_FLAG_STAT_RG	(t31_io_baddr+3)
#define	T31_FLAG_SEL_RG		(t31_io_baddr+0x403)


#define T31_MRESET_RESET 	0x0		// Control Register bits
#define T31_MRESET_GO		0x1
#define T31_MRESET_BITMASK	0x1

#define T31_SB_GLOCK_BITMASK	0x4
#define T31_SB_LOCK_BITMASK	0x8

#define T31_SB_CCL_BYTE		0
#define T31_SB_CCL_W16		0x10
#define T31_SB_CCL_W32		0x20
#define T31_SB_CCL_W32_2	0x30
#define T31_SB_CCL_BITMASK	0x30

#define T31_SB_ERR_IE_BITMASK	0x80
#define T31_MH_RQ_IE_BITMASK	0x40


#define T31_SYS_STAT_FRG	0x0		// Flag registers
#define T31_SYS_STAT_FRG_0	0x0
#define T31_SYS_STAT_FRG_1	0x10
#define T31_SYS_STAT_FRG_2	0x20
#define T31_SYS_STAT_FRG_3	0x30

#define T31_SET_HM_RQ_FRG	0x10
#define T31_CLR_MH_RQ_FRG	0x20
#define T31_CLR_SB_ERR_FRG	0x30
#define T31_T31_FCNTR_FRG	0x00

#define T31_ISA_MI_BADDR_FRG	0xE0

#define T31_DEV_ID0_FRG		0xF0
#define T31_DEV_ID1_FRG		0xF1


#define T31_SB_ACK_FLAG_BITMASK		0x1		// SYS_STATUS_FRG Flag Status register bits
#define T31_SB_ERR_FLAG_BITMASK		0x80
#define T31_MH_RQ_FLAG_BITMASK		0x40

#define T31_CLR_MH_RQ_FLAG	0x20		// T31 Flag Sel register bits
#define T31_CLR_SB_ERR_FLAG	0x30
#define T31_HM_RQ_FLAG		0x10


//-------------------------------------------------------------------------



//-------------------- macros for CONTROL REGISTER ---------------------------

#define T31_CNTR_RG_DATA	inp(T31_CNTR_RG) 

#define T31_MRESET_DATA		(inp(T31_CNTR_RG) & T31_MRESET_BITMASK) 
#define T31_SET_MRESET		outp(T31_CNTR_RG,inp(T31_CNTR_RG)&~T31_MRESET_BITMASK) 
#define T31_SET_MGO		outp(T31_CNTR_RG,inp(T31_CNTR_RG)&~T31_MRESET_BITMASK | T31_MRESET_GO) 

#define T31_SB_CCL_DATA		(inp(T31_CNTR_RG) & T31_SB_CCL_BITMASK)
#define T31_SET_SB_CCL_BYTE	outp(T31_CNTR_RG,inp(T31_CNTR_RG)&~T31_SB_CCL_BITMASK |T31_SB_CCL_BYTE)
#define T31_SET_SB_CCL_W16	outp(T31_CNTR_RG,inp(T31_CNTR_RG)&~T31_SB_CCL_BITMASK |T31_SB_CCL_W16)
#define T31_SET_SB_CCL_W32	outp(T31_CNTR_RG,inp(T31_CNTR_RG)&~T31_SB_CCL_BITMASK |T31_SB_CCL_W32)

#define T31_SB_GLOCK_DATA	(inp(T31_CNTR_RG) &  T31_SB_GLOCK_BITMASK)
#define T31_SET_SB_GLOCK_ON	outp(T31_CNTR_RG,inp(T31_CNTR_RG) |  T31_SB_GLOCK_BITMASK)
#define T31_SET_SB_GLOCK_OFF	outp(T31_CNTR_RG,inp(T31_CNTR_RG) & ~T31_SB_GLOCK_BITMASK)
#define T31_SB_LOCK_DATA	(inp(T31_CNTR_RG) &  T31_SB_LOCK_BITMASK)
#define T31_SET_SB_LOCK_ON	outp(T31_CNTR_RG,inp(T31_CNTR_RG) |  T31_SB_LOCK_BITMASK)
#define T31_SET_SB_LOCK_OFF	outp(T31_CNTR_RG,inp(T31_CNTR_RG) & ~T31_SB_LOCK_BITMASK)

#define T31_SB_ERR_IE_DATA	(inp(T31_CNTR_RG) &  T31_SB_ERR_IE_BITMASK)
#define T31_SET_SB_ERR_IE_ON	outp(T31_CNTR_RG,inp(T31_CNTR_RG) |  T31_SB_ERR_IE_BITMASK)
#define T31_SET_SB_ERR_IE_OFF	outp(T31_CNTR_RG,inp(T31_CNTR_RG) & ~T31_SB_ERR_IE_BITMASK)

#define T31_MH_RQ_IE_DATA	(inp(T31_CNTR_RG) &  T31_MH_RQ_IE_BITMASK)
#define T31_SET_MH_RQ_IE_ON	outp(T31_CNTR_RG,inp(T31_CNTR_RG) |  T31_MH_RQ_IE_BITMASK)
#define T31_SET_MH_RQ_IE_OFF	outp(T31_CNTR_RG,inp(T31_CNTR_RG) & ~T31_MH_RQ_IE_BITMASK)

#define T31_INIT		{outp(T31_CNTR_RG,T31_MRESET_RESET);T31_clr_sb_err();T31_clr_mh_rq();}



//-------------------- macros for UECM control ---------------------------

#define UECM_SELECT_MDSP	outp(uecm_io_baddr+0x800,0)
#define UECM_SELECT_XDSP	outp(uecm_io_baddr+0x801,0)
#define UECM_RESET		outp(uecm_io_baddr+0x818,0)



//------------------ macros for FLAGs control/status --------------------

#define T31_FLAG_SEL_RG_DATA		inp(T31_FLAG_SEL_RG)
#define T31_SET_FLAG_SEL_RG(n)		outp(T31_FLAG_SEL_RG,n)
#define	T31_SET_FLAG_SEL_RG_VERIF(n)	T31_set_flag_sel_rg_verif(n)

#define T31_FLAG_STAT_RG_DATA		inp(T31_FLAG_STAT_RG)
#define T31_SET_FLAG_CNTR_RG(n)		outp(T31_FLAG_CNTR_RG,n)

#define T31_SET_FLAG_RG_VERIF(r,v)	T31_set_flag_rg_verif(r,v)
#define T31_FLAG_RG_DATA(r)		T31_flag_rg_data(r)


#define T31_SYS_STAT_FRG_DATA	T31_sys_stat_frg_data()
#define T31_FLAG_STATUS_RG_DATA	T31_sys_stat_frg_data()
#define T31_SB_ERR_DATA		T31_sb_err_data()
#define T31_SB_ACK_DATA		T31_sb_ack_data()
#define T31_MH_RQ_DATA		T31_mh_rq_data()


#define T31_SET_HM_RQ		T31_set_hm_rq()
#define T31_CLR_SB_ERR		T31_clr_sb_err()
#define T31_CLR_MH_RQ		T31_clr_mh_rq()


#define T31_SET_HM_RQ		T31_set_hm_rq()
#define T31_CLR_SB_ERR		T31_clr_sb_err()
#define T31_CLR_MH_RQ		T31_clr_mh_rq()


int	T31_set_flag_sel_rg_verif(unsigned frg);
int	T31_set_flag_rg_verif(unsigned frg,BYTE value);
BYTE	T31_flag_rg_data(unsigned frg);
BYTE	T31_sys_stat_frg_data();
int	T31_sb_err_data();
int	T31_sb_ack_data();
int	T31_mh_rq_data();
int	T31_set_hm_rq();
int	T31_clr_sb_err();
int	T31_clr_mh_rq();

#define __MK_FP__(seg,ofs)	((void far *)(((unsigned long)(seg) << 16) | (unsigned)(ofs)))


#define T31_SET_SMP_RG(page)	outpw(T31_SMP_RG,page)
#define T31_SMP_RG_DATA		inpw(T31_SMP_RG)

#define T31_MAKE_SMP(saddr)		((unsigned)( (long)(saddr/T31_SMP_LEN)))
#define T31_MAKE_SMP_PTR(saddr)		(__MK_FP__(t31_smp_segm,(unsigned)((long)(saddr%T31_SMP_LEN))))

#define T31_SET_SMP_SEGM(s)	T31_set_smp_segm(s)
#define T31_SET_SMP_OFF		T31_set_smp_off()

int	T31_set_smp_segm(unsigned segm);
int	T31_set_smp_off();
int	T31_smp_ptr(unsigned long addr,unsigned *page,void **p);
int	T31_set_smp_ptr(unsigned long addr,void **p);

//------------------ macros/function for DEV_ID access --------------------

int	T31_dev_id_data(unsigned *dev_id);


//========================================================================


