/***************************************************************************

  TORNADO-31 Control Utilities. v.2.0
  (for Borland C++ v.3.0, Microsoft C v.5.10)

  (C) MicroLAB Systems Ltd, 1993-1994

****************************************************************************/


#include <conio.h>


#define	T31_OK			0		// error codes
#define	T31_PARAM_ERR		-1


//================= T31 specific definitions ==============================


#define T31_DSP_EADDR		0xffffffL  // C31 address range
#define T31_DSP_SADDR		0L
#define T31_DSP_ON_CHIP_SADDR	0x800000L  // C31 on-chip memory address range
#define T31_DSP_ON_CHIP_EADDR	0x809FFFL

#define T31_SB_EADDR		0xffffffL  // SB address range
#define T31_SB_SADDR		0L

#define T31_SRAM_EADDR		0x1fffffL  // SRAM base/end address
#define T31_SRAM_SADDR		0
#define T31_SRAM_LEN_MAX	0x200000L  // total SRAM length

#define T31_PIOX_SADDR		0xF00000L  // PIOX base/end address
#define T31_PIOX_EADDR		0xFFFFFFL
#define T31_PIOX_LEN_MAX	0x100000L



unsigned t31_smp_segm		= 0xd800;	// T31 SMP Window parameters
unsigned t31_io_baddr		= 0x350;        // T31 ISA I/O base address
unsigned t31_uecm_io_baddr	= 0x240;


#define  T31_SMP_LEN		0x8000

#define	 T31_SMP_RG		(t31_io_baddr)  // T31 I/O registers
#define  T31_SMP_RG_LSB		(t31_io_baddr)
#define  T31_SMP_RG_MSB		(t31_io_baddr+1)
#define  T31_CNTR_RG		(t31_io_baddr+2)
#define  T31_FLAG_CNTR_RG	(t31_io_baddr+3)
#define  T31_FLAG_STAT_RG	(t31_io_baddr+3)
#define  T31_FLAG_SEL_RG	(t31_io_baddr+0x403)


#define T31_MRESET_RESET 	0x0		// Control Register bits
#define T31_MRESET_GO		0x1
#define T31_MRESET_UECM		0x2
#define T31_MRESET_RESET_2	0x3
#define T31_MRESET_BITMASK	0x3

#define T31_SB_GLOCK_BITMASK	0x4
#define T31_SB_LOCK_BITMASK	0x8

#define T31_SB_CCL_BYTE		0
#define T31_SB_CCL_W16		0x10
#define T31_SB_CCL_W32		0x20
#define T31_SB_CCL_W32_2	0x30
#define T31_SB_CCL_BITMASK	0x30

#define T31_SB_ERR_IE_BITMASK	0x80
#define T31_MH_RQ_IE_BITMASK	0x40


#define T31_SB_ACK_FLAG_BITMASK	0x1		// Flag Status register bits
#define T31_SB_ERR_FLAG_BITMASK	0x80
#define T31_MH_RQ_FLAG_BITMASK	0x40

#define T31_CLR_MH_RQ_FLAG	0x20		// Flag Control register bits
#define T31_CLR_SB_ERR_FLAG	0x30
#define T31_HM_RQ_FLAG		0x10

#define T31_MCLOCK		20000000.00	// DSP clock freq in Hz

//-------------------------------------------------------------------------



//------------------------------ macros -----------------------------------

#define __MK_FP__(seg,ofs)	((void far *)(((unsigned long)(seg) << 16) | (unsigned)(ofs)))

#define T31_SET_SMP_RG(page)	outpw(T31_SMP_RG,page)
#define T31_SMP_RG_DATA		inpw(T31_SMP_RG)
#define T31_MAKE_SMP(saddr)	((unsigned)( (long)(saddr/T31_SMP_LEN)))
#define T31_MAKE_SMP_PTR(saddr)	(__MK_FP__(t31_smp_segm,(unsigned)((long)(saddr%T31_SMP_LEN))))

#define T31_INIT		{outpw(T31_CNTR_RG,T31_MRESET_RESET|(T31_SB_ERR_CLR__<<8));T31_CLR_MH_RQ;}

#define T31_CNTR_RG_DATA	inp(T31_CNTR_RG)

#define T31_MRESET_DATA		(inp(T31_CNTR_RG) & T31_MRESET_BITMASK)
#define T31_SET_MRESET		outp(T31_CNTR_RG,inp(T31_CNTR_RG)&~T31_MRESET_BITMASK)
#define T31_SET_MGO		outp(T31_CNTR_RG,inp(T31_CNTR_RG)&~T31_MRESET_BITMASK | T31_MRESET_GO)
#define T31_SET_MRESET_UECM	outp(T31_CNTR_RG,inp(T31_CNTR_RG)&~T31_MRESET_BITMASK | T31_MRESET_UECM)

#define T31_SB_CCL_DATA		(inp(T31_CNTR_RG) & T31_SB_CCL_BITMASK)
#define T31_SET_SB_CCL_BYTE	outp(T31_CNTR_RG,inp(T31_CNTR_RG)&~T31_SB_CCL_BITMASK |T31_SB_CCL_BYTE)
#define T31_SET_SB_CCL_W16	outp(T31_CNTR_RG,inp(T31_CNTR_RG)&~T31_SB_CCL_BITMASK |T31_SB_CCL_W16)
#define T31_SET_SB_CCL_W32	outp(T31_CNTR_RG,inp(T31_CNTR_RG)&~T31_SB_CCL_BITMASK |T31_SB_CCL_W32)

#define T31_SB_GLOCK_DATA	(inp(T31_CNTR_RG) &  T31_SB_GLOCK_BITMASK)
#define T31_SET_SB_GLOCK_ON	outp(T31_CNTR_RG,inp(T31_CNTR_RG) |  T31_SB_GLOCK_BITMASK)
#define T31_SET_SB_GLOCK_OFF	outp(T31_CNTR_RG,inp(T31_CNTR_RG) & ~T31_SB_GLOCK_BITMASK)
#define T31_SB_LOCK_DATA	(inp(T31_CNTR_RG) &  T31_SB_LOCK_BITMASK)
#define T31_SET_SB_LOCK_ON	outp(T31_CNTR_RG,inp(T31_CNTR_RG) |  T31_SB_LOCK_BITMASK)
#define T31_SET_SB_LOCK_OFF	outp(T31_CNTR_RG,inp(T31_CNTR_RG) & ~T31_SB_LOCK_BITMASK)

#define T31_SB_ERR_IE_DATA	(inp(T31_CNTR_RG) &  T31_SB_ERR_IE_BITMASK)
#define T31_SET_SB_ERR_IE_ON	outp(T31_CNTR_RG,inp(T31_CNTR_RG) |  T31_SB_ERR_IE_BITMASK)
#define T31_SET_SB_ERR_IE_OFF	outp(T31_CNTR_RG,inp(T31_CNTR_RG) & ~T31_SB_ERR_IE_BITMASK)

#define T31_MH_RQ_IE_DATA	(inp(T31_CNTR_RG) &  T31_MH_RQ_IE_BITMASK)
#define T31_SET_MH_RQ_IE_ON	outp(T31_CNTR_RG,inp(T31_CNTR_RG) |  T31_MH_RQ_IE_BITMASK)
#define T31_SET_MH_RQ_IE_OFF	outp(T31_CNTR_RG,inp(T31_CNTR_RG) & ~T31_MH_RQ_IE_BITMASK)


#define T31_FLAG_STATUS_RG_DATA	(inp(T31_FLAG_STAT_RG) & (T31_SB_ERR_FLAG_BITMASK|T31_MH_RQ_FLAG_BITMASK|T31_SB_ACK_FLAG_BITMASK))

#define T31_SB_ERR_DATA		(inp(T31_FLAG_STAT_RG) & T31_SB_ERR_FLAG_BITMASK)
#define T31_MH_RQ_DATA		(inp(T31_FLAG_STAT_RG) & T31_MH_RQ_FLAG_BITMASK)
#define T31_SB_ACK_DATA		(inp(T31_FLAG_STAT_RG) & T31_SB_ACK_FLAG_BITMASK)

#define T31_SET_HM_RQ		{outp(T31_FLAG_SEL_RG,T31_HM_RQ_FLAG);outp(T31_FLAG_CNTR_RG,0);outp(T31_FLAG_SEL_RG,0);}
#define T31_CLR_SB_ERR		{outp(T31_FLAG_SEL_RG,T31_CLR_SB_ERR_FLAG);outp(T31_FLAG_CNTR_RG,0);outp(T31_FLAG_SEL_RG,0);}
#define T31_CLR_MH_RQ		{outp(T31_FLAG_SEL_RG,T31_CLR_MH_RQ_FLAG);outp(T31_FLAG_CNTR_RG,0);outp(T31_FLAG_SEL_RG,0);}

#define T31_UECM_SELECT_MDSP	outp(t31_uecm_io_baddr+0x800,0)
#define T31_UECM_SELECT_XDSP	outp(t31_uecm_io_baddr+0x801,0)
#define T31_UECM_RESET		outp(t31_uecm_io_baddr+0x818,0)




int T31_smp_ptr(unsigned long addr,unsigned *page,void **p)
     // convert SRAM ADDR to *PAGE SMP_RG value & PC RAM **P pointer.

  {  
     if((addr/4L)>T31_SB_EADDR)   return(T31_PARAM_ERR) ;
		 *page=T31_MAKE_SMP(addr);
     *p=T31_MAKE_SMP_PTR(addr);
     return(T31_OK);
   }


int T31_set_smp_ptr(unsigned long addr,void **p)
     // Set SMP RG to the specified SRAM address.
     // Set PC SMP pointer to the *P.

  {  unsigned page;

     if(T31_smp_ptr(addr,&page,p)!=T31_OK) return(T31_PARAM_ERR);
     T31_SET_SMP_RG(page);
     return(T31_OK);
   }

//========================================================================

