/***************************************************************************

  TORNADO-31M Control Utilities. v.1.0
  (C) MicroLAB Systems Ltd, 1994-1997

  note: this file include definitions and function declarations only
	(should be used in conjunction with T3M.C file in order to
         keep definitions and vars/functions separately in a library manner)

****************************************************************************/


#include <conio.h>

typedef unsigned char BYTE ; 


#define	T3M_OK				0		// error codes
#define	T3M_PARAM_ERR			-1
#define	T3M_SMP_OFF_ERR			-3
#define	T3M_HW_ERR			-5
#define	T3M_NOT_T31_1A_FCNTR_FRG_ERR	-10


//================= T32 specific definitions ==============================


#define T3M_DSP_EADDR		0xffffffL  // C3x address range
#define T3M_DSP_SADDR		0L
#define T3M_DSP_ON_CHIP_SADDR	0x800000L  // C3x on-chip memory address range
#define T3M_DSP_ON_CHIP_EADDR	0x809fffL

#define T3M_SB_EADDR		0xfffffL  // SB address range
#define T3M_SB_SADDR		0L

#define T3M_SRAM_EADDR		0xfffffL  // SRAM base/end address
#define T3M_SRAM_SADDR		0
#define T3M_SRAM_LEN_MAX	0x100000L  // total SRAM length



//------------------- T31M ISA I/O I/F ---------------------- 

extern	unsigned	t3m_io_baddr;        // T31M ISA I/O base address
extern	unsigned	T3M_IO_BADDR_LIST[4];
#define	T3M_IO_BADDR_CNF_QTY	4


//------------------- ECC ---------------------- 

extern	unsigned	ecc_io_baddr;	// ECC I/O base address
extern	unsigned	ECC_IO_BADDR_LIST[8];

#define	ECC_IO_BADDR_CNF_QTY	8




//------------------- T32 ISA-CNF & SMP I/F ---------------------- 

#define	T3M_SMP_OFF		0


extern	unsigned	t3m_smp_segm;		// T32 SMP Window parameters
#define	t3m_smp_len	0x8000L			// SMP length
#define	T3M_SMP_LEN	0x8000L			// SMP length

extern	unsigned	T3M_SMP_SEGM_LIST[8]		;
#define T3M_SMP_SEGM_CNF_QTY		8


//------------------- T32 I/O registers ---------------------- 

#define	T3M_SMP_RG		(t3m_io_baddr)
#define	T3M_SMP_RG_LSB		(t3m_io_baddr) 
#define	T3M_SMP_RG_MSB		(t3m_io_baddr+1) 
#define	T3M_CNTR_RG		(t3m_io_baddr+2)
#define	T3M_FLAG_DATA_RG	(t3m_io_baddr+3)
#define	T3M_FLAG_CNTR_RG	(t3m_io_baddr+3)
#define	T3M_FLAG_STAT_RG	(t3m_io_baddr+3)
#define	T3M_FLAG_SEL_RG		(t3m_io_baddr+0x403)


#define T3M_MRESET_RESET 	0x0		// Control Register bits
#define T3M_MRESET_GO		0x1
#define T3M_MRESET_BITMASK	0x1

#define T3M_SB_GLOCK_BITMASK	0x4
#define T3M_SB_LOCK_BITMASK	0x8

#define T3M_SB_CCL_BYTE		0
#define T3M_SB_CCL_W16		0x10
#define T3M_SB_CCL_W32		0x20
#define T3M_SB_CCL_W32_2	0x30
#define T3M_SB_CCL_BITMASK	0x30

#define T3M_SB_ERR_IE_BITMASK	0x80
#define T3M_MH_RQ_IE_BITMASK	0x40


#define T3M_SYS_STAT_FRG	0x0		// Flag registers
#define T3M_SYS_STAT_FRG_0	0x0
#define T3M_SYS_STAT_FRG_1	0x10
#define T3M_SYS_STAT_FRG_2	0x20
#define T3M_SYS_STAT_FRG_3	0x30

#define T3M_SET_HM_RQ_FRG	0x10
#define T3M_CLR_MH_RQ_FRG	0x20
#define T3M_CLR_SB_ERR_FRG	0x30
#define T3M_T31_FCNTR_FRG	0x00

#define T3M_ISA_MI_BADDR_FRG	0xE0

#define T3M_ECC_IO_BADDR_FRG	0xE2

#define T3M_DEV_ID0_FRG		0xF0
#define T3M_DEV_ID1_FRG		0xF1


#define T3M_SB_ACK_FLAG_BITMASK		0x1		// SYS_STATUS_FRG Flag Status register bits
#define T3M_SB_ERR_FLAG_BITMASK		0x80
#define T3M_MH_RQ_FLAG_BITMASK		0x40

#define T31_CLR_MH_RQ_FLAG	0x20		// T31 Flag Sel register bits
#define T31_CLR_SB_ERR_FLAG	0x30
#define T31_HM_RQ_FLAG		0x10

//-------------------------------------------------------------------------



//-------------------- macros for CONTROL REGISTER ---------------------------

#define T3M_CNTR_RG_DATA	inp(T3M_CNTR_RG) 

#define T3M_MRESET_DATA		(inp(T3M_CNTR_RG) & T3M_MRESET_BITMASK) 
#define T3M_SET_MRESET		outp(T3M_CNTR_RG,inp(T3M_CNTR_RG)&~T3M_MRESET_BITMASK) 
#define T3M_SET_MGO		outp(T3M_CNTR_RG,inp(T3M_CNTR_RG)&~T3M_MRESET_BITMASK | T3M_MRESET_GO) 

#define T3M_SB_CCL_DATA		(inp(T3M_CNTR_RG) & T3M_SB_CCL_BITMASK)
#define T3M_SET_SB_CCL_BYTE	outp(T3M_CNTR_RG,inp(T3M_CNTR_RG)&~T3M_SB_CCL_BITMASK |T3M_SB_CCL_BYTE)
#define T3M_SET_SB_CCL_W16	outp(T3M_CNTR_RG,inp(T3M_CNTR_RG)&~T3M_SB_CCL_BITMASK |T3M_SB_CCL_W16)
#define T3M_SET_SB_CCL_W32	outp(T3M_CNTR_RG,inp(T3M_CNTR_RG)&~T3M_SB_CCL_BITMASK |T3M_SB_CCL_W32)

#define T3M_SB_GLOCK_DATA	(inp(T3M_CNTR_RG) &  T3M_SB_GLOCK_BITMASK)
#define T3M_SET_SB_GLOCK_ON	outp(T3M_CNTR_RG,inp(T3M_CNTR_RG) |  T3M_SB_GLOCK_BITMASK)
#define T3M_SET_SB_GLOCK_OFF	outp(T3M_CNTR_RG,inp(T3M_CNTR_RG) & ~T3M_SB_GLOCK_BITMASK)
#define T3M_SB_LOCK_DATA	(inp(T3M_CNTR_RG) &  T3M_SB_LOCK_BITMASK)
#define T3M_SET_SB_LOCK_ON	outp(T3M_CNTR_RG,inp(T3M_CNTR_RG) |  T3M_SB_LOCK_BITMASK)
#define T3M_SET_SB_LOCK_OFF	outp(T3M_CNTR_RG,inp(T3M_CNTR_RG) & ~T3M_SB_LOCK_BITMASK)

#define T3M_SB_ERR_IE_DATA	(inp(T3M_CNTR_RG) &  T3M_SB_ERR_IE_BITMASK)
#define T3M_SET_SB_ERR_IE_ON	outp(T3M_CNTR_RG,inp(T3M_CNTR_RG) |  T3M_SB_ERR_IE_BITMASK)
#define T3M_SET_SB_ERR_IE_OFF	outp(T3M_CNTR_RG,inp(T3M_CNTR_RG) & ~T3M_SB_ERR_IE_BITMASK)

#define T3M_MH_RQ_IE_DATA	(inp(T3M_CNTR_RG) &  T3M_MH_RQ_IE_BITMASK)
#define T3M_SET_MH_RQ_IE_ON	outp(T3M_CNTR_RG,inp(T3M_CNTR_RG) |  T3M_MH_RQ_IE_BITMASK)
#define T3M_SET_MH_RQ_IE_OFF	outp(T3M_CNTR_RG,inp(T3M_CNTR_RG) & ~T3M_MH_RQ_IE_BITMASK)

#define T3M_INIT		{outp(T3M_CNTR_RG,T3M_MRESET_RESET);T3M_clr_sb_err();T3M_clr_mh_rq();}



//-------------------- macros for ECC control ---------------------------

int T3M_set_ecc_io_baddr(unsigned io_addr);
int T3M_set_ecc_off();

#define ECC_SELECT_MDSP(baddr)	(T3M_set_ecc_io_baddr(baddr))
#define ECC_SELECT_MDSP_DEFAULT	(T3M_set_ecc_io_baddr(0x240))
#define ECC_SELECT_XDSP		(T3M_set_ecc_off())
#define ECC_RESET		outp(ecc_io_baddr+0x818,0)




//------------------ macros for FLAGs control/status --------------------

#define T3M_FLAG_SEL_RG_DATA		inp(T3M_FLAG_SEL_RG)
#define T3M_SET_FLAG_SEL_RG(n)		outp(T3M_FLAG_SEL_RG,n)
#define	T3M_SET_FLAG_SEL_RG_VERIF(n)	T3M_set_flag_sel_rg_verif(n)

#define T3M_FLAG_STAT_RG_DATA		inp(T3M_FLAG_STAT_RG)
#define T3M_SET_FLAG_CNTR_RG(n)		outp(T3M_FLAG_CNTR_RG,n)

#define T3M_SET_FLAG_RG_VERIF(r,v)	T3M_set_flag_rg_verif(r,v)
#define T3M_FLAG_RG_DATA(r)		T3M_flag_rg_data(r)


#define T3M_SYS_STAT_FRG_DATA	T3M_sys_stat_frg_data()
#define T3M_SB_ERR_DATA		T3M_sb_err_data()
#define T3M_SB_ACK_DATA		T3M_sb_ack_data()
#define T3M_MH_RQ_DATA		T3M_mh_rq_data()


#define T3M_SET_HM_RQ		T3M_set_hm_rq()
#define T3M_CLR_SB_ERR		T3M_clr_sb_err()
#define T3M_CLR_MH_RQ		T3M_clr_mh_rq()

#define T31_FLAG_STATUS_RG_DATA	(T3M_sys_stat_frg_data())
#define T31_SB_ERR_DATA		(T3M_sb_err_data() <<7 )
#define T31_MH_RQ_DATA		(T3M_mh_rq_data() <<6  )
#define T31_SB_ACK_DATA		(T3M_sb_ack_data())

#define T31_SET_HM_RQ		T3M_set_hm_rq()
#define T31_CLR_SB_ERR		T3M_clr_sb_err()
#define T31_CLR_MH_RQ		T3M_clr_mh_rq()


int	T3M_set_flag_sel_rg_verif(unsigned frg);
int	T3M_set_flag_rg_verif(unsigned frg,BYTE value);
BYTE	T3M_flag_rg_data(unsigned frg);
BYTE	T3M_sys_stat_frg_data();
int	T3M_sb_err_data();
int	T3M_sb_ack_data();
int	T3M_mh_rq_data();
int	T3M_set_hm_rq();
int	T3M_clr_sb_err();
int	T3M_clr_mh_rq();

#define __MK_FP__(seg,ofs)	((void far *)(((unsigned long)(seg) << 16) | (unsigned)(ofs)))


#define T3M_SET_SMP_RG(page)	outpw(T3M_SMP_RG,page & 0xf)
#define T3M_SMP_RG_DATA		(inpw(T3M_SMP_RG) & 0xf)

#define T3M_MAKE_SMP(saddr)		((unsigned)( (long)(saddr/T3M_SMP_LEN)))
#define T3M_MAKE_SMP_PTR(saddr)		(__MK_FP__(t3m_smp_segm,(unsigned)((long)(saddr%T3M_SMP_LEN))))

#define T3M_SET_SMP_SEGM(s)	T3M_set_smp_segm(s)
#define T3M_SET_SMP_OFF		T3M_set_smp_off()

int	T3M_set_smp_segm(unsigned segm);
int	T3M_set_smp_off();
int	T3M_smp_ptr(unsigned long addr,unsigned *page,void **p);
int	T3M_set_smp_ptr(unsigned long addr,void **p);

//------------------ macros/function for DEV_ID access --------------------

int	T3M_dev_id_data(unsigned *dev_id);


//========================================================================


