/***************************************************************************

  TORNADO-3X Host Control Utilities. v.3.2
  (C) MicroLAB Systems Ltd, 1993-2000

  note: this file include definitions and function declarations only
	and should be used together with T3X_H.C file.

****************************************************************************/

#define	T3X_ON				1
#define	T3X_OFF				0

typedef unsigned char BYTE ; 

#define	T3X_OK				0		// error codes
#define	T3X_PARAM_ERR			-1
#define	T3X_SMP_OFF_ERR			-3
#define	T3X_HW_ERR			-5
#define	T3X_NOT_OPEN_ERR		-10
#define	T3X_NOT_FOUND_ERR		-11
#define T3X_INV_CMND_ERR		-12
#define T3X_INV_SB_ADDR_ERR		-13


//================= T3x specific definitions ==============================

#define T3X_SRAM_SADDR		0L
#define T31_SRAM_EADDR		0x1fffffL  	// T31 4 banks 512Kx32 each
#define T31Z_SRAM_EADDR		0xfffffL  	// T31Z 2 banks 512Kx32 each
#define T31M_SRAM_EADDR		0xfffffL	// T31M 2 banks 512Kx32 each
#define T33_SRAM_EADDR		0xfffffL	// T33 2 banks 512Kx32 each
#define T3X_SRAM_EADDR_MAX	0x1fffffL  	// max T3X SRAM-eaddr
#define T3X_SRAM_LEN_MAX	0x200000L  	// total SRAM length

#define T3X_PIOX_SADDR		0xc00000L       // T3x 32-bit PIOX
#define T31_PIOX_SADDR		0xc00000L
#define T33_PIOX_SADDR		0xf00000L
#define T31_PIOX16_SADDR	0xff0000L
#define T3X_PIOX16_SADDR	0xff0000L
#define T31_PIOX_EADDR		0xffffffL  	
#define T33_PIOX_EADDR		0xffffffL  	
#define T3X_PIOX_EADDR_MAX	0xffffffL  	// max T31 IO-eaddr

#define T3X_SB_EADDR		0xffffffL  	// SB address range
#define T3X_SB_SADDR		0L


	//----- define C3x DSP environment params
#define T3X_DSP_EADDR		0xffffffL 	// C3x address range
#define T3X_DSP_SADDR		0L
#define T3X_DSP_ONCHIP_SADDR	0x800000L  	// C3x on-chip memory address range
#define T3X_DSP_ONCHIP_EADDR	0x809FFFL
#define T3X_DSP_PIOX_SADDR	0xC00000L  	// T3x PIOX area
#define T3X_DSP_PIOX_EADDR	0xFFFFFFL


	//----- T3x host i/f defs
#define	T31_DEV_ID		0x0100
#define	T31Z_DEV_ID		0x2400
#define	T31M_DEV_ID		0x5100
#define	T31MX_DEV_ID		0x5200
#define	T32L_DEV_ID		0x2600
#define	T33_DEV_ID		0x0400

#define	T31_IO_BADDR_CNF_QTY	8
#define	T33_IO_BADDR_CNF_QTY	8
#define	T31Z_IO_BADDR_CNF_QTY	16
#define	T31M_IO_BADDR_CNF_QTY	4

#define	T3X_SMP_SEGM_CNF_QTY	8
#define	T3X_IO_BADDR_DEFAULT	0x350
#define	T3X_SMP_SEGM_DEFAULT	0xd800
#define T3X_SMP_LEN		0x8000
#define	T3X_SMP_OFF		0


unsigned T3X_io_baddr();
unsigned T3X_smp_segm();
unsigned T3X_id();
unsigned T3X_emu_io_baddr();
unsigned T3X_io_baddr_table(int);
unsigned T3X_smp_segm_table(int);
unsigned T3X_ecc_io_baddr_table(int);
unsigned T3X_uecm_io_baddr_table(int);


//------------------- T6X I/O registers ---------------------- 
#define	 T3X_SMP_RG			(T3X_io_baddr())  // T6X I/O registers
#define  T3X_SMP_RG_LSB			(T3X_io_baddr())
#define  T3X_SMP_RG_MSB			(T3X_io_baddr()+1)
#define  T3X_CNTR_RG			(T3X_io_baddr()+2)
#define  T3X_FDATA_RG			(T3X_io_baddr()+3)
#define  T3X_FSEL_RG			(T3X_io_baddr()+0x403)

#define T3X_MRESET_RESET 		0x0		// Control Register bits
#define T3X_MRESET_GO			0x1
#define T3X_MRESET_BITMASK		0x1

#define T3X_SB_GLOCK_BITMASK		0x4
#define T3X_SB_LOCK_BITMASK		0x8

#define T3X_SB_CCL_BYTE			0
#define T3X_SB_CCL_W16			0x10
#define T3X_SB_CCL_W32			0x20
#define T3X_SB_CCL_W32_2		0x30
#define T3X_SB_CCL_BITMASK		0x30

#define T3X_SB_ERR_IE_BITMASK		0x80
#define T3X_MH_RQ_IE_BITMASK		0x40

#define T3X_SB_ACK_FLAG_BITMASK		0x1		// SYSTEM STATUS FRG bits
#define T3X_SB_ERR_FLAG_BITMASK		0x80
#define T3X_MH_RQ_FLAG_BITMASK		0x40

#define T3X_DSP_MLOCK_BITMASK		0x1	// DSP status Flag RG
#define T3X_DSP_MRESET_BITMASK		0x2

#define T3X_DSP_PXSX_SFT_RESET_EN_BITMASK	0x1	// COMPAT Flag RG

		//--- FLAG registers
#define T3X_SYS_STAT_FRG		0x10		
#define T3X_CLR_MH_RQ_FRG		0x20		
#define T3X_CLR_SB_ERR_FRG		0x30
#define T3X_DSP_STAT_FRG		0x60
#define T3X_SET_HM_RQ_FRG		0x10
#define T3X_ISA_MI_BADDR_FRG		0xE0
#define T3X_ECC_IO_BADDR_FRG		0xE2
#define T3X_COMPAT_FRG			0xE4
#define T3X_DEV_ID0_FRG			0xF0
#define T3X_DEV_ID1_FRG			0xF1
#define T3X_SYS_STAT_FRG_0		0x0		// other flag rgs
#define T3X_SYS_STAT_FRG_1		0x10
#define T3X_SYS_STAT_FRG_2		0x20
#define T3X_SYS_STAT_FRG_3		0x30
#define T3X_T31R1_FCNTR_FRG		0x00

//-------------------------------------------------------------------------



//----------------------- macros for CONTROL REGISTER -----------------------------

#define T3X_CNTR_RG_DATA		inp(T3X_CNTR_RG)
#define T3X_SET_CNTR_RG(d)		outp(T3X_CNTR_RG,d)

#define T3X_MRESET_DATA			(inp(T3X_CNTR_RG) & T3X_MRESET_BITMASK)
#define T3X_SET_MRESET			outp(T3X_CNTR_RG,inp(T3X_CNTR_RG)&~T3X_MRESET_BITMASK)
#define T3X_SET_MGO			outp(T3X_CNTR_RG,inp(T3X_CNTR_RG)&~T3X_MRESET_BITMASK | T3X_MRESET_GO)

#define T3X_SB_GLOCK_DATA		(inp(T3X_CNTR_RG) &  T3X_SB_GLOCK_BITMASK)
#define T3X_SET_SB_GLOCK_ON		outp(T3X_CNTR_RG,inp(T3X_CNTR_RG) |  T3X_SB_GLOCK_BITMASK)
#define T3X_SET_SB_GLOCK_OFF		outp(T3X_CNTR_RG,inp(T3X_CNTR_RG) & ~T3X_SB_GLOCK_BITMASK)
#define T3X_SB_LOCK_DATA		(inp(T3X_CNTR_RG) &  T3X_SB_LOCK_BITMASK)
#define T3X_SET_SB_LOCK_ON		outp(T3X_CNTR_RG,inp(T3X_CNTR_RG) |  T3X_SB_LOCK_BITMASK)
#define T3X_SET_SB_LOCK_OFF		outp(T3X_CNTR_RG,inp(T3X_CNTR_RG) & ~T3X_SB_LOCK_BITMASK)

#define T3X_SB_CCL_DATA			(inp(T3X_CNTR_RG) & T3X_SB_CCL_BITMASK)
#define T3X_SET_SB_CCL_BYTE		outp(T3X_CNTR_RG,inp(T3X_CNTR_RG)&~T3X_SB_CCL_BITMASK |T3X_SB_CCL_BYTE)
#define T3X_SET_SB_CCL_W16		outp(T3X_CNTR_RG,inp(T3X_CNTR_RG)&~T3X_SB_CCL_BITMASK |T3X_SB_CCL_W16)
#define T3X_SET_SB_CCL_W32		outp(T3X_CNTR_RG,inp(T3X_CNTR_RG)&~T3X_SB_CCL_BITMASK |T3X_SB_CCL_W32)

#define T3X_SB_ERR_IE_DATA		(inp(T3X_CNTR_RG) &  T3X_SB_ERR_IE_BITMASK)
#define T3X_SET_SB_ERR_IE_ON		outp(T3X_CNTR_RG,inp(T3X_CNTR_RG) |  T3X_SB_ERR_IE_BITMASK)
#define T3X_SET_SB_ERR_IE_OFF		outp(T3X_CNTR_RG,inp(T3X_CNTR_RG) & ~T3X_SB_ERR_IE_BITMASK)

#define T3X_MH_RQ_IE_DATA		(inp(T3X_CNTR_RG) &  T3X_MH_RQ_IE_BITMASK)
#define T3X_SET_MH_RQ_IE_ON		outp(T3X_CNTR_RG,inp(T3X_CNTR_RG) |  T3X_MH_RQ_IE_BITMASK)
#define T3X_SET_MH_RQ_IE_OFF		outp(T3X_CNTR_RG,inp(T3X_CNTR_RG) & ~T3X_MH_RQ_IE_BITMASK)

#define T3X_INIT			{outpw(T3X_CNTR_RG,0);outpw(T3X_SMP_RG,0);T3X_clr_sb_err();}


//------------------ macros for FLAGs control/status --------------------

int	T3X_set_fsel_rg_verif(unsigned frg);
int	T3X_set_frg_verif(unsigned frg,BYTE value);
BYTE	T3X_frg_data(unsigned frg);
BYTE	T3X_sys_stat_frg_data();
int	T3X_sb_err_data();
int	T3X_sb_ack_data();
int	T3X_mh_rq_data();
int	T3X_set_hm_rq();
int	T3X_clr_sb_err();
int	T3X_clr_mh_rq();
int	T3X_compat_frg_data();
int	T3X_dsp_pxsx_sft_reset_en_data();
int	T3X_set_dsp_pxsx_sft_reset_en(unsigned i);
int	T3X_dsp_stat_frg_data();
int	T3X_dsp_mlock_data();
int	T3X_dsp_mreset_data();


#define T3X_FSEL_RG_DATA		inp(T3X_FSEL_RG)
#define T3X_SET_FSEL_RG(n)		outp(T3X_FSEL_RG,n)
#define	T3X_SET_FSEL_RG_VERIF(n)	T3X_set_fsel_rg_verif(n)

#define T3X_FDATA_RG_DATA		inp(T3X_FDATA_RG)
#define T3X_SET_FDATA_RG(n)		outp(T3X_FDATA_RG,n)

#define T3X_SET_FRG_VERIF(r,v)		T3X_set_frg_verif(r,v)
#define T3X_FRG_DATA(r)			T3X_frg_data(r)

#define T3X_SYS_STAT_FRG_DATA		T3X_sys_stat_frg_data()
#define T3X_SB_ERR_DATA			T3X_sb_err_data()
#define T3X_SB_ACK_DATA			T3X_sb_ack_data()
#define T3X_MH_RQ_DATA			T3X_mh_rq_data()

#define T3X_SET_HM_RQ			T3X_set_hm_rq()
#define T3X_CLR_SB_ERR			T3X_clr_sb_err()
#define T3X_CLR_MH_RQ			T3X_clr_mh_rq()

#define	T3X_COMPAT_FRG_DATA		T3X_compat_frg_data()
#define	T3X_DSP_PXSX_SFT_RESET_EN_DATA	T3X_dsp_pxsx_sft_reset_en_data()
#define	T3X_SET_DSP_PXSX_SFT_RESET_EN(i)	T3X_set_dsp_pxsx_sft_reset_en(i)

#define T3X_DSP_STAT_FRG_DATA		T3X_dsp_stat_frg_data()
#define T3X_DSP_MLOCK_DATA		T3X_dsp_mlock_data()
#define T3X_DSP_MRESET_DATA		T3X_dsp_mreset_data()

//==================== macros for SMP ===============================

#define T3X_SET_SMP_RG(page)		outpw(T3X_SMP_RG,page)
#define T3X_SMP_RG_DATA			inpw(T3X_SMP_RG)

#define T3X_SET_SMP_SEGM(s)		T3X_set_smp_segm(s)
#define T3X_SET_SMP_OFF			T3X_set_smp_off()

#define __MK_FP__(seg,offs)		((void far *)(((unsigned long)(seg) << 16) | (unsigned)(offs)))
#define T3X_MAKE_SMP(saddr)		((unsigned)( (long)(saddr/T3X_SMP_LEN)))
#define T3X_MAKE_SMP_PTR(saddr)	(__MK_FP__(T3X_smp_segm(),(unsigned)((long)(saddr%T3X_SMP_LEN))))


int	T3X_set_smp_segm(unsigned segm);
int	T3X_set_smp_off();
int	T3X_smp_ptr(unsigned long addr,unsigned *page,void **p);
int	T3X_set_smp_ptr(unsigned long addr,void **p);
int 	T3X_validate_sb_addr(unsigned long addr);

//------------------ macros/function for DEV_ID access --------------------

int	T3X_dev_id_data(unsigned *dev_id);
int 	T3X_dev_name(char *);

#define	T3X_dev_id(s)	T3X_dev_id_data(s)
#define	T31_BOARD	(T3X_id()==T31_DEV_ID)
#define	T31Z_BOARD	(T3X_id()==T31Z_DEV_ID)
#define	T31M_BOARD	(T3X_id()==T31M_DEV_ID)
#define	T33_BOARD	(T3X_id()==T33_DEV_ID)

#define T3X_VALID_BOARD	(T3X_id()==T31_DEV_ID || T3X_id()==T31Z_DEV_ID || T3X_id()==T31M_DEV_ID || T3X_id()==T33_DEV_ID)
#define T3X_BOARD	T3X_VALID_BOARD


//------------------- UECM/ECC ----------------------
#define	T3X_EMU_OFF			0
#define	T3X_EMU_IO_BADDR_DEFAULT	0x240
#define	T3X_ECC_IO_BADDR_CNF_QTY	8
#define	T3X_UECM_IO_BADDR_CNF_QTY	4
#define T3X_EMU_MDSP			0
#define T3X_EMU_XDSP			1


int	T3X_emu_io_baddr_data();
int 	T3X_set_emu_io_baddr(unsigned io_addr);
int 	T3X_set_emu_mdsp();
int 	T3X_set_emu_xdsp();
int 	T3X_set_emu_reset();
int 	T3X_emu_path_data();

#define T3X_set_emu_off() 		(T3X_set_emu_xdsp())
#define T3X_set_ecc_off() 		(T3X_set_emu_xdsp())
#define T3X_ECC_OFF	 		(T3X_set_emu_xdsp())
#define T3X_EMU(baddr)			(T3X_set_emu_io_baddr(baddr))
#define T3X_EMU_DEFAULT			(T3X_set_emu_io_baddr(T3X_EMU_IO_BADDR_DEFAULT))
#define T3X_EMU_SELECT_MDSP		(T3X_set_emu_mdsp())
#define T3X_EMU_SELECT_XDSP		(T3X_set_emu_xdsp())
#define T3X_EMU_RESET			(T3X_set_emu_reset())


//------------------- utility functions -----------------------------
int T3X_open(unsigned);
int T3X_close();
int T3X_last_err();
int T3X_err_flag();
int T3X_clr_err_flag();
int T3X_err_message(int,char *);
int T3X_last_err_message(char *);

